import system_tests


class ThrowsWhenSubBoxLengthIsNotGood(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/742"

    filename = system_tests.path("$data_path/issue_742_poc")
    commands = ["$exiv2 -pX $filename"]
    stdout = [""]
    stderr = ["""$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class MetadataPiping(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1137"

    filename = system_tests.path("$data_path/exiv2-empty_copy.jpg")
    Stonehenge = system_tests.path("$data_path/Stonehenge.exv")

    commands = [
        """$exiv2 -pa                   $filename""",
        """$exiv2 -PkV --grep GPSL      $Stonehenge""",
        """$exiv2 -m- $filename""",
        """$exiv2 -pa  --grep GPSL      $filename"""
    ]

    output_grep_GPSL = """set Exif.GPSInfo.GPSLatitudeRef                   N
set Exif.GPSInfo.GPSLatitude                      51/1 106969/10000 0/1
set Exif.GPSInfo.GPSLongitudeRef                  W
set Exif.GPSInfo.GPSLongitude                     1/1 495984/10000 0/1
"""

    stdin = [
        None,
        None,
        output_grep_GPSL,
        None
    ]

    stdout = [
        "",
        output_grep_GPSL,
        "",
        """Exif.GPSInfo.GPSLatitudeRef                  Ascii       2  North
Exif.GPSInfo.GPSLatitude                     Rational    3  51deg 11' 0"
Exif.GPSInfo.GPSLongitudeRef                 Ascii       2  West
Exif.GPSInfo.GPSLongitude                    Rational    3  1deg 50' 0"
"""
    ]



# -*- coding: utf-8 -*-

import system_tests


class IPTCtimeWithoutTimeZone(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/440"

    filename = "$data_path/exiv2-bug440.jpg"
    commands = ["$exiv2 -u -b -pi $filename"]
    stdout = ["""Iptc.Application2.RecordVersion              Short       1  1
Iptc.Application2.ObjectName                 String     16  WHITE SOX TIGERS
Iptc.Application2.Category                   String      1  S
Iptc.Application2.SuppCategory               String      3  BBA
Iptc.Application2.DateCreated                Date        8  2005-09-29
Iptc.Application2.TimeCreated                Time       11  16:51:00+00:00
Iptc.Application2.Byline                     String     11  PAUL SANCYA
Iptc.Application2.BylineTitle                String      3  STF
Iptc.Application2.City                       String      7  DETROIT
Iptc.Application2.ProvinceState              String      2  MI
Iptc.Application2.CountryName                String      3  USA
Iptc.Application2.TransmissionReference      String      6  DTS112
Iptc.Application2.Headline                   String     14  GARCIA GARLAND
Iptc.Application2.Credit                     String      2  AP
Iptc.Application2.Source                     String      2  AP
Iptc.Application2.Caption                    String    282  Chicago White Sox pitcher John Garland, left, pours champagne on Freddy Garcia in the locker room after their 4-2 win against the Detroit Tigers in Detroit, Thursday, Sept. 29, 2005. With the win the White Sox clinched their first AL Central title since 2000. (AP Photo/Paul Sancya)
Iptc.Application2.Writer                     String     15  DD PCS DD**NY**
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class Issue1305Test(metaclass=system_tests.CaseMeta):
    err_msg_dir_img = """Warning: Directory Image, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Image, entry 0x3030 has invalid size 808464432*1; skipping entry.
"""

    err_msg_dir_ph = """Warning: Directory Photo, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Photo, entry 0x3030 has invalid size 808464432*1; skipping entry.
"""

    err_msg_dir_pentax = """Warning: Directory Pentax, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Pentax, entry 0x3030 has invalid size 808464432*1; skipping entry.
"""

    name = "regression test for issue 1305"
    url = "http://dev.exiv2.org/issues/1305"

    filename = "$data_path/IMGP0006-min.jpg"
    commands = ["$exiv2 $filename"]
    stdout = ["""File name       : $filename
File size       : 12341 Bytes
MIME type       : image/jpeg
Image size      : 0 x 0
Thumbnail       : None
Camera make     : PENTAX000000000000000000000000000000000000000000
Camera model    : 
Image timestamp : 
File number     : 
Exposure time   : 
Aperture        : 
Exposure bias   : 
Flash           : 
Flash bias      : 
Focal length    : 
Subject distance: 
ISO speed       : 
Exposure mode   : 
Metering mode   : 
Macro mode      : 
Image quality   : 
White balance   : 
Copyright       : 
Exif comment    : 

"""]

    stderr = [
        """Error: Directory Image: Next pointer is out of bounds; ignored.
"""
        + 8 * err_msg_dir_img
        + """Warning: Directory Photo has an unexpected next pointer; ignored.
"""
        + 13 * err_msg_dir_ph
        + """Warning: Directory Photo, entry 0x927c has unknown Exif (TIFF) type 12336; setting type size 1.
Warning: Directory Pentax has an unexpected next pointer; ignored.
"""
        + 6 * err_msg_dir_pentax
        + """Warning: Directory Pentax, entry 0x0006 has unknown Exif (TIFF) type 12336; setting type size 1.
Warning: Directory Pentax, entry 0x0007 has unknown Exif (TIFF) type 12336; setting type size 1.
"""
        + 39 * err_msg_dir_pentax
        + 23 * err_msg_dir_ph
        + """Warning: Directory Photo, entry 0x3030 has unknown Exif (TIFF) type 48; setting type size 1.
Error: Directory Photo, entry 0x3030 has invalid size 808464432*1; skipping entry.
"""
        + 5 * err_msg_dir_ph
        + """Warning: Directory Photo, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Photo, entry 0x3030 has invalid size 808452102*1; skipping entry.
Warning: Directory Photo, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Photo, entry 0x3030 has invalid size 808452103*1; skipping entry.
"""
        + 3 * err_msg_dir_ph
        + err_msg_dir_img
        + """Warning: Directory Image, entry 0x3030 has unknown Exif (TIFF) type 12336; setting type size 1.
Error: Directory Image, entry 0x3030 has invalid size 1414415696*1; skipping entry.
"""
        + 36 * err_msg_dir_img
        + """Warning: JPEG format error, rc = 5
"""]

    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class TestFirstPoC(metaclass=system_tests.CaseMeta):
    """
    Regression test for the first bug described in:
    https://github.com/Exiv2/exiv2/issues/253
    """
    url = "https://github.com/Exiv2/exiv2/issues/253"

    filename = "$data_path/3-stringformat-outofbound-read"
    commands = ["$exiv2 " + filename]
    stdout = [""]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerNotAJpeg
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class CheckNikonTimezoneWithoutCruft(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1062"

    filename = system_tests.path("$data_path/exiv2-bug1062.jpg")
    commands = [ "$exiv2 -pa -g zone $filename" ]

    stdout = [ """Exif.NikonWt.Timezone                        SShort      1  UTC -06:00
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


URL = "http://dev.exiv2.org/issues/711"

COMMANDS = [
    """$exiv2 -u -v -M"set Exif.Image.ProcessingSoftware Initial values, read from the command line" -M"set Exif.Image.DocumentName Float 0.12345" -M"set Exif.Image.ImageDescription Double 0.987654321" $filename""",
      "$exiv2 -u -v -b -PEkyct $filename",
    """$exiv2 -u -v -M"set Exif.Image.ProcessingSoftware Non-intrusive update" $filename""",
      "$exiv2 -u -v -b -PEkyct $filename",
    """$exiv2 -u -v -M"set Exif.Image.ProcessingSoftware Intrusive update, writing the structure from scratch" $filename""",
      "$exiv2 -u -v -b -PEkyct $filename",
]

@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class LittleEndianORFFilesFromE_PL1Corrupted(metaclass=system_tests.CaseMeta):

    filename = "$data_path/exiv2-empty_copy.jpg"

    commands = COMMANDS

    stdout = [
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Initial values, read from the command line" (Ascii)
Set Exif.Image.DocumentName "0.12345" (Float)
Set Exif.Image.ImageDescription "0.987654321" (Double)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      43  Initial values, read from the command line
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
""",
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Non-intrusive update" (Ascii)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      21  Non-intrusive update
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
""",
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Intrusive update, writing the structure from scratch" (Ascii)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      53  Intrusive update, writing the structure from scratch
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
"""
    ]
    stderr = [""] * 6
    retval = [0] * 6


@system_tests.CopyFiles("$data_path/exiv2-kodak-dc210.jpg")
class BigEndianORFFilesFromE_PL1Corrupted(metaclass=system_tests.CaseMeta):

    filename = "$data_path/exiv2-kodak-dc210_copy.jpg"

    commands = COMMANDS

    stdout = [
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Initial values, read from the command line" (Ascii)
Set Exif.Image.DocumentName "0.12345" (Float)
Set Exif.Image.ImageDescription "0.987654321" (Double)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      43  Initial values, read from the command line
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
Exif.Image.Make                              Ascii      22  Eastman Kodak Company
Exif.Image.Model                             Ascii      20  DC210 Zoom (V05.00)
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  216
Exif.Image.YResolution                       Rational    1  216
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.Copyright                         Ascii       0  
Exif.Image.ExifTag                           Long        1  268
Exif.Photo.ExposureTime                      Rational    1  1/30 s
Exif.Photo.FNumber                           Rational    1  F4
Exif.Photo.ExifVersion                       Undefined   4  1.10
Exif.Photo.DateTimeOriginal                  Ascii      20  2000:10:26 16:46:51
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.CompressedBitsPerPixel            Rational    1  (0/0)
Exif.Photo.ShutterSpeedValue                 SRational   1  1/32 s
Exif.Photo.ApertureValue                     Rational    1  F4
Exif.Photo.BrightnessValue                   SRational   1  1.5
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MaxApertureValue                  Rational    1  F4
Exif.Photo.SubjectDistance                   Rational    1  Unknown
Exif.Photo.MeteringMode                      Short       1  Center weighted average
Exif.Photo.LightSource                       Short       1  Unknown
Exif.Photo.Flash                             Short       1  Fired
Exif.Photo.FocalLength                       Rational    1  4.4 mm
Exif.Photo.MakerNote                         Undefined  72  1 4 3 0 2 1 255 255 0 1 55 142 14 93 109 246 1 0 0 251 0 4 255 32 0 0 40 0 0 0 12 53 68 67 80 48 49 48 49 49 46 74 80 71 0 4 0 0 0 1 228 224 0 1 0 0 0 1 145 128 0 1 194 64 0 1 0 0 0 1 222 16
Exif.Thumbnail.ImageWidth                    Short       1  96
Exif.Thumbnail.ImageLength                   Short       1  72
Exif.Thumbnail.BitsPerSample                 Short       3  8 8 8
Exif.Thumbnail.Compression                   Short       1  Uncompressed
Exif.Thumbnail.PhotometricInterpretation     Short       1  RGB
Exif.Thumbnail.StripOffsets                  Short       1  822
Exif.Thumbnail.SamplesPerPixel               Short       1  3
Exif.Thumbnail.RowsPerStrip                  Short       1  72
Exif.Thumbnail.StripByteCounts               Short       1  20736
Exif.Thumbnail.XResolution                   Rational    1  72
Exif.Thumbnail.YResolution                   Rational    1  72
Exif.Thumbnail.ResolutionUnit                Short       1  inch
""",
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Non-intrusive update" (Ascii)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      21  Non-intrusive update
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
Exif.Image.Make                              Ascii      22  Eastman Kodak Company
Exif.Image.Model                             Ascii      20  DC210 Zoom (V05.00)
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  216
Exif.Image.YResolution                       Rational    1  216
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.Copyright                         Ascii       0  
Exif.Image.ExifTag                           Long        1  268
Exif.Photo.ExposureTime                      Rational    1  1/30 s
Exif.Photo.FNumber                           Rational    1  F4
Exif.Photo.ExifVersion                       Undefined   4  1.10
Exif.Photo.DateTimeOriginal                  Ascii      20  2000:10:26 16:46:51
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.CompressedBitsPerPixel            Rational    1  (0/0)
Exif.Photo.ShutterSpeedValue                 SRational   1  1/32 s
Exif.Photo.ApertureValue                     Rational    1  F4
Exif.Photo.BrightnessValue                   SRational   1  1.5
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MaxApertureValue                  Rational    1  F4
Exif.Photo.SubjectDistance                   Rational    1  Unknown
Exif.Photo.MeteringMode                      Short       1  Center weighted average
Exif.Photo.LightSource                       Short       1  Unknown
Exif.Photo.Flash                             Short       1  Fired
Exif.Photo.FocalLength                       Rational    1  4.4 mm
Exif.Photo.MakerNote                         Undefined  72  1 4 3 0 2 1 255 255 0 1 55 142 14 93 109 246 1 0 0 251 0 4 255 32 0 0 40 0 0 0 12 53 68 67 80 48 49 48 49 49 46 74 80 71 0 4 0 0 0 1 228 224 0 1 0 0 0 1 145 128 0 1 194 64 0 1 0 0 0 1 222 16
Exif.Thumbnail.ImageWidth                    Short       1  96
Exif.Thumbnail.ImageLength                   Short       1  72
Exif.Thumbnail.BitsPerSample                 Short       3  8 8 8
Exif.Thumbnail.Compression                   Short       1  Uncompressed
Exif.Thumbnail.PhotometricInterpretation     Short       1  RGB
Exif.Thumbnail.StripOffsets                  Short       1  822
Exif.Thumbnail.SamplesPerPixel               Short       1  3
Exif.Thumbnail.RowsPerStrip                  Short       1  72
Exif.Thumbnail.StripByteCounts               Short       1  20736
Exif.Thumbnail.XResolution                   Rational    1  72
Exif.Thumbnail.YResolution                   Rational    1  72
Exif.Thumbnail.ResolutionUnit                Short       1  inch
""",
        """File 1/1: $filename
Set Exif.Image.ProcessingSoftware "Intrusive update, writing the structure from scratch" (Ascii)
""",
        """File 1/1: $filename
Exif.Image.ProcessingSoftware                Ascii      53  Intrusive update, writing the structure from scratch
Exif.Image.DocumentName                      Float       1  0.123450003564358
Exif.Image.ImageDescription                  Double      1  0.987654321
Exif.Image.Make                              Ascii      22  Eastman Kodak Company
Exif.Image.Model                             Ascii      20  DC210 Zoom (V05.00)
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  216
Exif.Image.YResolution                       Rational    1  216
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.Copyright                         Ascii       0  
Exif.Image.ExifTag                           Long        1  278
Exif.Photo.ExposureTime                      Rational    1  1/30 s
Exif.Photo.FNumber                           Rational    1  F4
Exif.Photo.ExifVersion                       Undefined   4  1.10
Exif.Photo.DateTimeOriginal                  Ascii      20  2000:10:26 16:46:51
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.CompressedBitsPerPixel            Rational    1  (0/0)
Exif.Photo.ShutterSpeedValue                 SRational   1  1/32 s
Exif.Photo.ApertureValue                     Rational    1  F4
Exif.Photo.BrightnessValue                   SRational   1  1.5
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MaxApertureValue                  Rational    1  F4
Exif.Photo.SubjectDistance                   Rational    1  Unknown
Exif.Photo.MeteringMode                      Short       1  Center weighted average
Exif.Photo.LightSource                       Short       1  Unknown
Exif.Photo.Flash                             Short       1  Fired
Exif.Photo.FocalLength                       Rational    1  4.4 mm
Exif.Photo.MakerNote                         Undefined  72  1 4 3 0 2 1 255 255 0 1 55 142 14 93 109 246 1 0 0 251 0 4 255 32 0 0 40 0 0 0 12 53 68 67 80 48 49 48 49 49 46 74 80 71 0 4 0 0 0 1 228 224 0 1 0 0 0 1 145 128 0 1 194 64 0 1 0 0 0 1 222 16
Exif.Thumbnail.ImageWidth                    Short       1  96
Exif.Thumbnail.ImageLength                   Short       1  72
Exif.Thumbnail.BitsPerSample                 Short       3  8 8 8
Exif.Thumbnail.Compression                   Short       1  Uncompressed
Exif.Thumbnail.PhotometricInterpretation     Short       1  RGB
Exif.Thumbnail.StripOffsets                  Short       1  832
Exif.Thumbnail.SamplesPerPixel               Short       1  3
Exif.Thumbnail.RowsPerStrip                  Short       1  72
Exif.Thumbnail.StripByteCounts               Short       1  20736
Exif.Thumbnail.XResolution                   Rational    1  72
Exif.Thumbnail.YResolution                   Rational    1  72
Exif.Thumbnail.ResolutionUnit                Short       1  inch
"""
    ]

    stderr = [""] * 6
    retval = [0] * 6



# -*- coding: utf-8 -*-

import system_tests

class NikonTamronLens_F045_35_F14(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/1069"

    filename = "$data_path/Tamron_SP_35mm_f1.4_Di_USD_F045.exv"
    commands = ["$exiv2 -pa --grep lensid/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.NikonLd3.LensIDNumber                   Byte        1  Tamron SP 35mm f/1.4 Di USD
"""
]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class TestCvePoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/58"

    filename = "$data_path/POC11"
    commands = ["$exiv2 $filename"]
    stdout = [
        """File name       : $filename
File size       : 100 Bytes
MIME type       : image/pgf
Image size      : 131345 x 65536
"""
    ]
    stderr = ["""Error: Directory Image: Next pointer is out of bounds; ignored.
Error: Directory Image, entry 0x0000 has invalid size 3402235904*1; skipping entry.
Error: Directory Image, entry 0x014a has invalid size 4294967295*1; skipping entry.
Warning: Directory Image, entry 0x014a doesn't look like a sub-IFD.
Warning: Directory Image, entry 0x4720 has unknown Exif (TIFF) type 60362; setting type size 1.
Error: Directory Image, entry 0x4720 has invalid size 1330792777*1; skipping entry.
Warning: Directory Image, entry 0x0001 has unknown Exif (TIFF) type 0; setting type size 1.
Error: Directory Image, entry 0x0001 has invalid size 3401632458*1; skipping entry.
$filename: No Exif data found in the file
"""]
    retval = ["$no_exif_data_found_retval"]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class OutOfBoundsReadInIptcParserDecode(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/867
    """
    url = "https://github.com/Exiv2/exiv2/issues/867"

    filename = path("$data_path/issue_867_poc.psd")
    commands = ["$exiv2 $filename"]
    stdout = ["""File name       : $filename
File size       : 9830 Bytes
MIME type       : image/x-photoshop
Image size      : 150 x 91
"""
]
    stderr = [
        """Warning: Failed to decode IPTC metadata.
$filename: No Exif data found in the file
"""
]
    retval = [253]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/issue_1845_poc.jp2")

class TiffDirectoryWriteDirEntryAssert(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1845
    """
    url = "https://github.com/Exiv2/exiv2/issues/1845"

    filename = path("$tmp_path/issue_1845_poc.jp2")
    commands = ["$exiv2 -q -D +1 ad $filename"]
    stderr = [""]
    stdout = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class Olympus_MZuiko_17mm_f12_Pro(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1375"
    
    filename = "$data_path/olympus-m.zuiko-17mm-f1.2-pro.exv"
    commands  = ["$exiv2 -pa -K Exif.OlympusEq.LensType $filename"]
    stderr = ["""Warning: Directory OlympusCs, entry 0x0101: Strip 0 is outside of the data area; ignored.
"""]
    stdout = ["""Exif.OlympusEq.LensType                      Byte        6  Olympus M.Zuiko Digital ED 17mm F1.2 Pro
"""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class PanicOnMisinformedIPTC(metaclass=system_tests.CaseMeta):

    encodings = ['latin-1']

    url = "http://dev.exiv2.org/issues/444"

    filename = "$data_path/exiv2-bug444.jpg"
    commands = ["$exiv2 -u -pi $filename"]
    stdout = ["""Iptc.Envelope.CharacterSet                   String      4  \x00Ä\x1c\x02
Iptc.Application2.Urgency                    String      2  GT
Iptc.Application2.SuppCategory               String      8  portrait
Iptc.Application2.Caption                    String     18  Witty caption here
Iptc.Application2.Keywords                   String      3  ize
Iptc.Application2.0x007b                     String      4  n123
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class NikonTamronLens_A030_70_300_F4_56(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/598"

    filename = "$data_path/Tamron_SP70-300_F4-5.6_Di_VC_USD_A030.exv"
    commands = ["$exiv2 -pa --grep lensid/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.NikonLd3.LensIDNumber                   Byte        1  Tamron SP 70-300mm F/4-5.6 Di VC USD
"""
]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class InvalidDateXMP(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1706
    """
    url = "https://github.com/Exiv2/exiv2/issues/1706"

    filename = path("$data_path/issue_1706_poc.exv")
    commands = ["$exiv2 -PE $filename"]

    stderr = [
"""Error: Directory Photo with 65280 entries considered invalid; not read.
"""
]
    retval = [0]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        # Check that it printed "Bad value" for the date.
        self.assertRegex(got_stdout, "Exif.PentaxDng.Date\\s+Long\\s+1\\s+Bad value")



# -*- coding: utf-8 -*-

import system_tests


class CanonLenses(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1252"
    filenames = [
        "$data_path/exiv2-bug1252a.exv",
        "$data_path/exiv2-bug1252b.exv",
    ]

    commands = [
        "$exiv2 -pa --grep lens/i " + filenames[0],
        "$exiv2 -pa --grep lens/i " + filenames[1],
    ]

    stdout = ["""Exif.CanonCs.LensType                        Short       1  Sigma APO 120-300mm f/2.8 EX DG OS HSM
Exif.CanonCs.Lens                            Short       3  120.0 - 300.0 mm
Exif.Canon.LensModel                         Ascii      74  120-300mm
Exif.Photo.LensSpecification                 Rational    4  120/1 300/1 0/1 0/1
Exif.Photo.LensModel                         Ascii      10  120-300mm
Exif.Photo.LensSerialNumber                  Ascii      11  0000000000
""",
              """Exif.CanonCs.LensType                        Short       1  Sigma 150-500mm f/5-6.3 APO DG OS HSM
Exif.CanonCs.Lens                            Short       3  150.0 - 500.0 mm
Exif.Canon.LensModel                         Ascii      74  150-500mm
Exif.Photo.LensSpecification                 Rational    4  150/1 500/1 0/1 0/1
Exif.Photo.LensModel                         Ascii      10  150-500mm
Exif.Photo.LensSerialNumber                  Ascii      11  0000000000
"""
    ]

    stderr = [""] * len(commands)
    retval = [0]  * len(commands)



# -*- coding: utf-8 -*-

import system_tests


class CheckBarometer(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1225"
    filename = "$data_path/exiv2-bug1225.exv"

    commands = [
        "$exiv2 -pa --grep Bar $filename"
    ]

    stdout = [
        "Exif.Nikon3.BarometerInfo                    Undefined  12  1583\n"
    ]

    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors

class Jp2ImagePrintStructureICC(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-583f-w9pm-99r2
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-583f-w9pm-99r2"

    filename = path("$data_path/issue_ghsa_583f_w9pm_99r2_poc.jp2")
    commands = ["$exiv2 -p C $filename"]
    stdout = [""]
    stderr = ["""Exiv2 exception in print action for file $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class DenialOfServiceInAdjustTimeOverflow(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/851

    The date parsing code in XMPUtils::ConvertToDate does not
    check that the month and day are in bounds. This can cause a
    denial of service in AdjustTimeOverflow because it adjusts
    out-of-bounds days in a loop that subtracts one month per
    iteration.
    """
    url = "https://github.com/Exiv2/exiv2/issues/851"

    filename = path("$data_path/issue_851_poc.xmp")
    commands = ["$exiv2 $filename"]
    stdout = ["""File name       : $filename
File size       : 317 Bytes
MIME type       : application/rdf+xml
Image size      : 0 x 0
"""
]
    stderr = ["""Error: XMP Toolkit error 201: Error in XMLValidator
Warning: Failed to decode XMP metadata.
$filename: No Exif data found in the file
"""]
    retval = [253]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class MemoryLeakInPngImagePrintStructure(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1817

    Note: the test only fails in an ASAN build.
    """
    url = "https://github.com/Exiv2/exiv2/issues/1817"

    filename = path("$data_path/issue_1817_poc.png")
    commands = ["$exiv2 -pS $filename"]
    stdout = ["""STRUCTURE OF PNG FILE: $filename
 address | chunk |  length | data                           | checksum
       8 | eXIf  |       0 |                                | 0x00000000
"""]
    stderr = ["""$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors


class TestCvePoC(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/283
    """
    url = "https://github.com/Exiv2/exiv2/issues/283"
    cve_url = "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-11531"

    def check_no_ASAN_UBSAN_errors(self, i, command, got_stderr, expected_stderr):
        """ Override of system_tests.check_no_ASAN_UBSAN_errors for this particular test case.

        Here we want to also check that the two last lines of got_stderr have the expected_stderr
        """

        check_no_ASAN_UBSAN_errors(self, i, command, got_stderr, expected_stderr)
        self.assertListEqual(expected_stderr.splitlines(), got_stderr.splitlines()[-2:])

    filename = path("$data_path/pocIssue283.jpg")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""]
    compare_stderr = check_no_ASAN_UBSAN_errors
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class TestPoC(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/168"

    stderr_common = """Error: Directory Image: IFD exceeds data buffer, cannot read next pointer.
Error: Offset of directory Image, entry 0x0117 is out of bounds: Offset = 0x30303030; truncating the entry
""" + 12 * """Error: Offset of directory Image, entry 0x3030 is out of bounds: Offset = 0x30303030; truncating the entry
"""

    filename = "$data_path/cve_2017_18005_reproducer.tiff"

    commands = [
        "$exiv2 -vPEIXxgklnycsvth -b " + filename,
        "$exiv2json " + filename
    ]

    stdout = ["""File 1/1: """ + filename + """
0x0117 Image        Exif.Image.StripByteCounts                   StripByteCounts             Strip Byte Count               SByte       0   0  

""",
    """{
	"Exif": {
		"Image": {
			"StripByteCounts": 0,
			"0x3030": 0,
			"0x3030": "",
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0,
			"0x3030": 0
		}
	}
}
"""
    ]
    stderr = [
        stderr_common + filename + """: No IPTC data found in the file
""" + filename + """: No XMP data found in the file
""",
        stderr_common
    ]
    retval = [0, 0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Sigma_Art_85mm_F1_4Nikon_F_mount(metaclass=CaseMeta):

    filename = path("$data_path/exiv2-issue1208.exv")
    commands = ["$exiv2 -pa --grep lens/i $filename"]

    stdout = ["""Exif.Nikon3.LensType                         Byte        1  D G 
Exif.Nikon3.Lens                             Rational    4  85mm F1.4
Exif.Nikon3.LensFStops                       Undefined   4  7
Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma 85mm F1.4 DG HSM | A
Exif.NikonLd3.LensFStops                     Byte        1  F7.0
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class ReadDataAfterTagDelete(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/498"

    filename = "$data_path/exiv2-empty_copy.jpg"

    commands = [
        """$exiv2 -u -v -M"set Exif.GPSInfo.GPSLatitude 0/1 1/1 2/1" $filename""",
        "$exiv2 -u -v -pv $filename",
        """$exiv2 -u -v -M"del Exif.GPSInfo.GPSLatitude" $filename""",
        "$exiv2 -u -v -pv $filename"
    ]

    stdout = [
        """File 1/1: $filename
Set Exif.GPSInfo.GPSLatitude "0/1 1/1 2/1" (Rational)
""",
        """File 1/1: $filename
0x8825 Image        GPSTag                      Long        1  26
0x0002 GPSInfo      GPSLatitude                 Rational    3  0/1 1/1 2/1
""",
        """File 1/1: $filename
Del Exif.GPSInfo.GPSLatitude
""",
        """File 1/1: $filename
"""
    ]

    stderr = [""] * 3 + ["""$filename: No Exif data found in the file
"""]
    retval = [0] * 4



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class Issue743NikonSigmaArtLens(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/743
    """
    url = "https://github.com/Exiv2/exiv2/issues/743"

    filename = path("$data_path/issue_743.exv")
    commands = ["$exiv2 -pa --grep lensid/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma 24-70mm F2.8 DG OS HSM Art
"""
]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class CheckLensDetectionTeleconverter(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1122"

    filename = system_tests.path("$data_path/exiv2-bug1122.exv")
    commands = [ "$exiv2 -pv -g Lens $filename",
                 "$exiv2 -pa -g Lens $filename",
               ]

    stdout = [ """0x0016 CanonCs      LensType                    Short       1  173
0x0017 CanonCs      Lens                        Short       3  1000 300 1
0x0095 Canon        LensModel                   Ascii      74  300-1000mm
0xa432 Photo        LensSpecification           Rational    4  300/1 1000/1 0/1 0/1
0xa434 Photo        LensModel                   Ascii      11  300-1000mm
0xa435 Photo        LensSerialNumber            Ascii      11  0000000000
""",
    """Exif.CanonCs.LensType                        Short       1  Sigma 150-500mm f/5-6.3 APO DG OS HSM + 2x
Exif.CanonCs.Lens                            Short       3  300.0 - 1000.0 mm
Exif.Canon.LensModel                         Ascii      74  300-1000mm
Exif.Photo.LensSpecification                 Rational    4  300/1 1000/1 0/1 0/1
Exif.Photo.LensModel                         Ascii      11  300-1000mm
Exif.Photo.LensSerialNumber                  Ascii      11  0000000000
"""
    ]
    stderr = [""] * len(commands)
    retval = [0] * len(commands)



#!/usr/bin/env python3

import sys
import csv

print("""<?xml version = '1.0'?>
<?xml-stylesheet type=\"text/xsl\" href=\"tags.xsl\"?>
<TAGLIST>
<HEADER>
<title>Iptc datasets defined in Exiv2</title>
<text>
<p>Datasets are defined according to the specification of the Iptc 
<a href=\"http://www.iptc.org/IIM/\">Information Interchange Model (IIM)</a>.</p>
<p>Click on a column header to sort the table.</p>
</text>
</HEADER>
<ROWSET>""")

row=0
data = sys.stdin.readlines()
for line in csv.reader(data,quotechar='"',skipinitialspace=True):
    row=row+1
    print("   <ROW num=\"%d\">" % row)
    print("      <tagname>"    + line[ 0] + "</tagname>")
    print("      <tagdec>"     + line[ 1] + "</tagdec>")
    print("      <taghex>"     + line[ 2] + "</taghex>")
    print("      <recname>"    + line[ 3] + "</recname>")
    print("      <mandatory>"  + line[ 4] + "</mandatory>")
    print("      <repeatable>" + line[ 5] + "</repeatable>")
    print("      <minbytes>"   + line[ 6] + "</minbytes>")
    print("      <maxbytes>"   + line[ 7] + "</maxbytes>")
    print("      <key>"        + line[ 8] + "</key>")
    print("      <type>"       + line[ 9] + "</type>")
    print("      <tagdesc>"    + line[10] + "</tagdesc>")
    print("   </ROW>")

print("</ROWSET>")
print("</TAGLIST>")




# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class DenialOfServiceInAdjustTimeOverflow(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/853

    The date parsing code in XMPUtils::ConvertToDate does not
    check that the month and day are in bounds. This can cause a
    denial of service in AdjustTimeOverflow because it adjusts
    out-of-bounds days in a loop that subtracts one month per
    iteration.
    """
    url = "https://github.com/Exiv2/exiv2/issues/853"

    filename = path("$data_path/issue_853_poc.jpg")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """Exiv2 exception in print action for file $filename:
Not a valid ICC Profile
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-bug445.jpg",
                        "$data_path/exiv2-empty.jpg")
class SetCommentOnImageWithEmptyComment(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/445"

    bug_repr = "$data_path/exiv2-bug445_copy.jpg"
    empty_jpg = "$data_path/exiv2-empty_copy.jpg"

    commands = [
        """$exiv2 -u -v -M"set Exif.Photo.UserComment A comment" $bug_repr""",
        "$exiv2 -u -b -pt $bug_repr",
        """$exiv2 -u -v -M"set Exif.Photo.UserComment A comment" $empty_jpg""",
        "$exiv2 -u -b -pt $empty_jpg"
    ]

    stdout = [
        """File 1/1: $bug_repr
Set Exif.Photo.UserComment "A comment" (Comment)
""",
        """Exif.Image.Make                              Ascii       6  Canon
Exif.Image.Model                             Ascii      20  Canon PowerShot S40
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.XResolution                       Rational    1  180
Exif.Image.YResolution                       Rational    1  180
Exif.Image.ResolutionUnit                    Short       1  inch
Exif.Image.DateTime                          Ascii      20  2003:12:14 12:01:44
Exif.Image.YCbCrPositioning                  Short       1  Centered
Exif.Image.ExifTag                           Long        1  196
Exif.Photo.ExposureTime                      Rational    1  1/500 s
Exif.Photo.FNumber                           Rational    1  F4.9
Exif.Photo.ExifVersion                       Undefined   4  2.20
Exif.Photo.DateTimeOriginal                  Ascii      20  2003:12:14 12:01:44
Exif.Photo.DateTimeDigitized                 Ascii      20  2003:12:14 12:01:44
Exif.Photo.ComponentsConfiguration           Undefined   4  YCbCr
Exif.Photo.CompressedBitsPerPixel            Rational    1  5
Exif.Photo.ShutterSpeedValue                 SRational   1  1/501 s
Exif.Photo.ApertureValue                     Rational    1  F5
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MaxApertureValue                  Rational    1  F2.8
Exif.Photo.MeteringMode                      Short       1  Center weighted average
Exif.Photo.Flash                             Short       1  No, auto
Exif.Photo.FocalLength                       Rational    1  21.3 mm
Exif.Photo.MakerNote                         Undefined 450  12 0 1 0 3 0 40 0 0 0 68 4 0 0 2 0 3 0 4 0 0 0 148 4 0 0 3 0 3 0 4 0 0 0 156 4 0 0 4 0 3 0 27 0 0 0 164 4 0 0 0 0 3 0 6 0 0 0 218 4 0 0 0 0 3 0 4 0 0 0 230 4 0 0 6 0 2 0 32 0 0 0 238 4 0 0 7 0 2 0 24 0 0 0 14 5 0 0 8 0 4 0 1 0 0 0 59 225 17 0 9 0 2 0 32 0 0 0 38 5 0 0 16 0 4 0 1 0 0 0 0 0 17 1 13 0 3 0 21 0 0 0 70 5 0 0 0 0 0 0 80 0 2 0 0 0 5 0 1 0 0 0 0 0 4 0 0 0 1 0 0 0 1 0 0 0 0 0 0 0 0 0 17 0 5 0 1 0 3 48 1 0 255 255 255 255 170 2 227 0 32 0 149 0 192 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 49 0 224 8 224 8 0 0 1 0 2 0 170 2 30 1 215 0 0 0 0 0 0 0 0 0 54 0 0 0 160 0 20 1 149 0 31 1 0 0 0 0 0 0 0 0 6 0 0 0 0 0 0 0 2 48 0 0 0 0 0 0 1 0 14 3 0 0 149 0 33 1 0 0 0 0 0 0 250 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 73 77 71 58 80 111 119 101 114 83 104 111 116 32 83 52 48 32 74 80 69 71 0 0 0 0 0 0 0 0 0 0 70 105 114 109 119 97 114 101 32 86 101 114 115 105 111 110 32 49 46 49 48 0 0 0 65 110 100 114 101 97 115 32 72 117 103 103 101 108 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 42 0 3 0 1 128 122 1 1 128 0 0 0 0 0 0 3 1 2 0 0 0 10 0 0 0 0 0 0 0 57 0 198 0 5 0 0 0 0 0 0 0
Exif.MakerNote.Offset                        Long        1  942
Exif.MakerNote.ByteOrder                     Ascii       3  II
Exif.CanonCs.0x0000                          Short       1  80
Exif.CanonCs.Macro                           Short       1  Off
Exif.CanonCs.Selftimer                       Short       1  Off
Exif.CanonCs.Quality                         Short       1  Superfine
Exif.CanonCs.FlashMode                       Short       1  Auto
Exif.CanonCs.DriveMode                       Short       1  Single / timer
Exif.CanonCs.0x0006                          Short       1  0
Exif.CanonCs.FocusMode                       Short       1  Single
Exif.CanonCs.0x0008                          Short       1  0
Exif.CanonCs.RecordMode                      Short       1  JPEG
Exif.CanonCs.ImageSize                       Short       1  Large
Exif.CanonCs.EasyMode                        Short       1  Manual
Exif.CanonCs.DigitalZoom                     Short       1  None
Exif.CanonCs.Contrast                        Short       1  Normal
Exif.CanonCs.Saturation                      Short       1  Normal
Exif.CanonCs.Sharpness                       Short       1  Normal
Exif.CanonCs.ISOSpeed                        Short       1  100
Exif.CanonCs.MeteringMode                    Short       1  Center-weighted average
Exif.CanonCs.FocusType                       Short       1  Auto
Exif.CanonCs.AFPoint                         Short       1  Center
Exif.CanonCs.ExposureProgram                 Short       1  Program (P)
Exif.CanonCs.0x0015                          Short       1  65535
Exif.CanonCs.LensType                        Short       1  n/a
Exif.CanonCs.Lens                            Short       3  7.1 - 21.3 mm
Exif.CanonCs.MaxAperture                     Short       1  F5
Exif.CanonCs.MinAperture                     Short       1  F8
Exif.CanonCs.FlashActivity                   Short       1  Did not fire
Exif.CanonCs.FlashDetails                    Short       1  
Exif.CanonCs.0x001e                          Short       1  0
Exif.CanonCs.0x001f                          Short       1  0
Exif.CanonCs.FocusContinuous                 Short       1  Single
Exif.CanonCs.AESetting                       Short       1  Normal AE
Exif.CanonCs.ImageStabilization              Short       1  (65535)
Exif.CanonCs.DisplayAperture                 Short       1  49
Exif.CanonCs.ZoomSourceWidth                 Short       1  2272
Exif.CanonCs.ZoomTargetWidth                 Short       1  2272
Exif.CanonCs.0x0026                          Short       1  0
Exif.CanonCs.SpotMeteringMode                Short       1  AF Point
Exif.Canon.FocalLength                       Short       4  21.3 mm
Exif.Canon.0x0003                            Short       4  0 0 0 0
Exif.CanonSi.0x0000                          Short       1  54
Exif.CanonSi.AutoISO                         Short       1  100
Exif.CanonSi.ISOSpeed                        Short       1  100
Exif.CanonSi.MeasuredEV                      Short       1  13.63
Exif.CanonSi.TargetAperture                  Short       1  F5
Exif.CanonSi.TargetShutterSpeed              Short       1  1/501 s
Exif.CanonSi.0x0006                          Short       1  0
Exif.CanonSi.WhiteBalance                    Short       1  Auto
Exif.CanonSi.SlowShutter                     Short       1  Off
Exif.CanonSi.Sequence                        Short       1  0
Exif.CanonSi.0x000a                          Short       1  6
Exif.CanonSi.0x000b                          Short       1  0
Exif.CanonSi.CameraTemperature               Short       1  --
Exif.CanonSi.FlashGuideNumber                Short       1  0
Exif.CanonSi.AFPointUsed                     Short       1  3 focus points; center used
Exif.CanonSi.FlashBias                       Short       1  0 EV
Exif.CanonSi.AutoExposureBracketing          Short       1  Off
Exif.CanonSi.0x0011                          Short       1  0
Exif.CanonSi.0x0012                          Short       1  1
Exif.CanonSi.SubjectDistance                 Short       1  7.82 m
Exif.CanonSi.0x0014                          Short       1  0
Exif.CanonSi.ApertureValue                   Short       1  F5
Exif.CanonSi.ShutterSpeedValue               Short       1  1/523 s
Exif.CanonSi.MeasuredEV2                     Short       1  -6.00
Exif.CanonSi.BulbDuration                    Short       1  0
Exif.CanonSi.0x0019                          Short       1  0
Exif.CanonSi.CameraType                      Short       1  Compact
Exif.Canon.0x0000                            Short       6  0 0 0 0 0 0
Exif.Canon.0x0000                            Short       4  0 0 0 0
Exif.Canon.ImageType                         Ascii      32  IMG:PowerShot S40 JPEG
Exif.Canon.FirmwareVersion                   Ascii      24  Firmware Version 1.10
Exif.Canon.FileNumber                        Long        1  117-1771
Exif.Canon.OwnerName                         Ascii      32  Andreas Huggel
Exif.Canon.ModelID                           Long        1  PowerShot S40
Exif.Canon.CameraInfo                        Short      21  42 3 32769 378 32769 0 0 0 259 2 0 10 0 0 0 57 198 5 0 0 0
Exif.Photo.UserComment                       Undefined  17  A comment
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ColorSpace                        Short       1  sRGB
Exif.Photo.PixelXDimension                   Short       1  2272
Exif.Photo.PixelYDimension                   Short       1  1704
Exif.Photo.InteroperabilityTag               Long        1  1416
Exif.Iop.InteroperabilityIndex               Ascii       4  R98
Exif.Iop.InteroperabilityVersion             Undefined   4  1.00
Exif.Iop.RelatedImageWidth                   Short       1  2272
Exif.Iop.RelatedImageLength                  Short       1  1704
Exif.Photo.FocalPlaneXResolution             Rational    1  8114.29
Exif.Photo.FocalPlaneYResolution             Rational    1  8114.29
Exif.Photo.FocalPlaneResolutionUnit          Short       1  inch
Exif.Photo.SensingMethod                     Short       1  One-chip color area
Exif.Photo.FileSource                        Undefined   1  Digital still camera
Exif.Photo.CustomRendered                    Short       1  Normal process
Exif.Photo.ExposureMode                      Short       1  Auto
Exif.Photo.WhiteBalance                      Short       1  Auto
Exif.Photo.DigitalZoomRatio                  Rational    1  1.0
Exif.Photo.SceneCaptureType                  Short       1  Standard
Exif.Thumbnail.Compression                   Short       1  JPEG (old-style)
Exif.Thumbnail.XResolution                   Rational    1  180
Exif.Thumbnail.YResolution                   Rational    1  180
Exif.Thumbnail.ResolutionUnit                Short       1  inch
Exif.Thumbnail.JPEGInterchangeFormat         Long        1  2036
Exif.Thumbnail.JPEGInterchangeFormatLength   Long        1  5448
""",
        """File 1/1: $empty_jpg
Set Exif.Photo.UserComment "A comment" (Comment)
""",
        """Exif.Image.ExifTag                           Long        1  26
Exif.Photo.UserComment                       Undefined  17  A comment
"""
    ]
    stderr = [""] * 4
    retval = [0] * 4



# -*- coding: utf-8 -*-

import system_tests


class ReadIPTfromJPEG(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/426"

    filename = "$data_path/exiv2-bug426.jpg"
    commands = ["$exiv2 -u -pi $filename"]
    stdout = ["""Iptc.Application2.RecordVersion              Short       1  2
Iptc.Application2.FixtureId                  String     22  2005 Oregon Coast Tour
Iptc.Application2.Byline                     String     14  Julie V. Early
Iptc.Application2.City                       String     21  159 km SW of Portland
Iptc.Application2.ProvinceState              String      6  Oregon
Iptc.Application2.CountryName                String     13  United States
Iptc.Application2.Contact                    String     16  Jeffrey J. Early
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests

class CheckCanonLenses(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1140"

    filename = system_tests.path("$data_path/exiv2-bug1140.exv")
    commands = [ "$exiv2 -pa -g Lens $filename" ]

    stdout = [ """Exif.CanonCs.LensType                        Short       1  Canon EF-S 24mm f/2.8 STM
Exif.CanonCs.Lens                            Short       3  24.0 mm
Exif.CanonCf.LensAFStopButton                Short       1  2304
Exif.Canon.LensModel                         Ascii      64  
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors

class PrintXmpDateOutOfBoundsIndex19(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1918
    """
    url = "https://github.com/Exiv2/exiv2/issues/1918"

    filename1 = path("$data_path/issue_1918_poc1.jpg")
    filename2 = path("$data_path/issue_1918_poc2.jpg")
    filename3 = path("$data_path/issue_1918_poc3.jpg")
    commands = ["$exiv2 -px $filename1", "$exiv2 -px $filename2", "$exiv2 -px $filename3"]
    stderr = ["", "", ""]
    retval = [0, 0, 0]

    compare_stdout = check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests
import unittest

# test needs system_tests.BT.vv['enable_bmff']=1
bSkip=system_tests.BT.verbose_version().get('enable_bmff')!='1'
if bSkip:
    raise unittest.SkipTest('*** requires enable_bmff=1 ***')

class pr_1475_2021_heic(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/2021-02-13-1929.heic"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2  -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS          $filename"
                   ,"$exiv2 -pX          $filename"
                   ,"$exiv2 -pC --binary $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.Make                              Ascii       6  Apple
Exif.Image.DateTime                          Ascii      20  2021:02:13 11:03:31
Exif.Photo.ExposureTime                      Rational    1  1/30 s
Exif.Photo.ExposureProgram                   Short       1  Auto
Exif.Photo.DateTimeOriginal                  Ascii      20  2021:02:13 11:03:31
Exif.Photo.DateTimeDigitized                 Ascii      20  2021:02:13 11:03:31
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ExposureMode                      Short       1  Auto
Exif.GPSInfo.GPSDateStamp                    Ascii      11  2021:02:13
""","""Exiv2::BmffImage::boxHandler: ftyp        0->36 brand: heic
Exiv2::BmffImage::boxHandler: meta       36->3380 
  Exiv2::BmffImage::boxHandler: hdlr       48->34 
  Exiv2::BmffImage::boxHandler: dinf       82->36 
  Exiv2::BmffImage::boxHandler: pitm      118->14 
  Exiv2::BmffImage::boxHandler: iinf      132->1085 
    Exiv2::BmffImage::boxHandler: infe      146->21 ID =   1 hvc1 
    Exiv2::BmffImage::boxHandler: infe      167->21 ID =   2 hvc1 
    Exiv2::BmffImage::boxHandler: infe      188->21 ID =   3 hvc1 
    Exiv2::BmffImage::boxHandler: infe      209->21 ID =   4 hvc1 
    Exiv2::BmffImage::boxHandler: infe      230->21 ID =   5 hvc1 
    Exiv2::BmffImage::boxHandler: infe      251->21 ID =   6 hvc1 
    Exiv2::BmffImage::boxHandler: infe      272->21 ID =   7 hvc1 
    Exiv2::BmffImage::boxHandler: infe      293->21 ID =   8 hvc1 
    Exiv2::BmffImage::boxHandler: infe      314->21 ID =   9 hvc1 
    Exiv2::BmffImage::boxHandler: infe      335->21 ID =  10 hvc1 
    Exiv2::BmffImage::boxHandler: infe      356->21 ID =  11 hvc1 
    Exiv2::BmffImage::boxHandler: infe      377->21 ID =  12 hvc1 
    Exiv2::BmffImage::boxHandler: infe      398->21 ID =  13 hvc1 
    Exiv2::BmffImage::boxHandler: infe      419->21 ID =  14 hvc1 
    Exiv2::BmffImage::boxHandler: infe      440->21 ID =  15 hvc1 
    Exiv2::BmffImage::boxHandler: infe      461->21 ID =  16 hvc1 
    Exiv2::BmffImage::boxHandler: infe      482->21 ID =  17 hvc1 
    Exiv2::BmffImage::boxHandler: infe      503->21 ID =  18 hvc1 
    Exiv2::BmffImage::boxHandler: infe      524->21 ID =  19 hvc1 
    Exiv2::BmffImage::boxHandler: infe      545->21 ID =  20 hvc1 
    Exiv2::BmffImage::boxHandler: infe      566->21 ID =  21 hvc1 
    Exiv2::BmffImage::boxHandler: infe      587->21 ID =  22 hvc1 
    Exiv2::BmffImage::boxHandler: infe      608->21 ID =  23 hvc1 
    Exiv2::BmffImage::boxHandler: infe      629->21 ID =  24 hvc1 
    Exiv2::BmffImage::boxHandler: infe      650->21 ID =  25 hvc1 
    Exiv2::BmffImage::boxHandler: infe      671->21 ID =  26 hvc1 
    Exiv2::BmffImage::boxHandler: infe      692->21 ID =  27 hvc1 
    Exiv2::BmffImage::boxHandler: infe      713->21 ID =  28 hvc1 
    Exiv2::BmffImage::boxHandler: infe      734->21 ID =  29 hvc1 
    Exiv2::BmffImage::boxHandler: infe      755->21 ID =  30 hvc1 
    Exiv2::BmffImage::boxHandler: infe      776->21 ID =  31 hvc1 
    Exiv2::BmffImage::boxHandler: infe      797->21 ID =  32 hvc1 
    Exiv2::BmffImage::boxHandler: infe      818->21 ID =  33 hvc1 
    Exiv2::BmffImage::boxHandler: infe      839->21 ID =  34 hvc1 
    Exiv2::BmffImage::boxHandler: infe      860->21 ID =  35 hvc1 
    Exiv2::BmffImage::boxHandler: infe      881->21 ID =  36 hvc1 
    Exiv2::BmffImage::boxHandler: infe      902->21 ID =  37 hvc1 
    Exiv2::BmffImage::boxHandler: infe      923->21 ID =  38 hvc1 
    Exiv2::BmffImage::boxHandler: infe      944->21 ID =  39 hvc1 
    Exiv2::BmffImage::boxHandler: infe      965->21 ID =  40 hvc1 
    Exiv2::BmffImage::boxHandler: infe      986->21 ID =  41 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1007->21 ID =  42 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1028->21 ID =  43 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1049->21 ID =  44 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1070->21 ID =  45 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1091->21 ID =  46 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1112->21 ID =  47 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1133->21 ID =  48 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1154->21 ID =  49 grid 
    Exiv2::BmffImage::boxHandler: infe     1175->21 ID =  50 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1196->21 ID =  51 Exif  *** Exif ***
  Exiv2::BmffImage::boxHandler: iref     1217->148 
  Exiv2::BmffImage::boxHandler: iprp     1365->1203 
    Exiv2::BmffImage::boxHandler: ipco     1373->876 
      Exiv2::BmffImage::boxHandler: colr     1381->560 
      Exiv2::BmffImage::boxHandler: hvcC     1941->112 
      Exiv2::BmffImage::boxHandler: ispe     2053->20 pixelWidth_, pixelHeight_ = 512, 512
      Exiv2::BmffImage::boxHandler: ispe     2073->20 pixelWidth_, pixelHeight_ = 4032, 3024
      Exiv2::BmffImage::boxHandler: irot     2093->9 
      Exiv2::BmffImage::boxHandler: pixi     2102->16 
      Exiv2::BmffImage::boxHandler: hvcC     2118->111 
      Exiv2::BmffImage::boxHandler: ispe     2229->20 pixelWidth_, pixelHeight_ = 320, 240
    Exiv2::BmffImage::boxHandler: ipma     2249->319 
  Exiv2::BmffImage::boxHandler: idat     2568->16 
  Exiv2::BmffImage::boxHandler: iloc     2584->832 
      2592 |       16 |   ID |    1 |  23265, 15453
      2608 |       16 |   ID |    2 |  38718, 21882
      2624 |       16 |   ID |    3 |  60600, 20196
      2640 |       16 |   ID |    4 |  80796, 22234
      2656 |       16 |   ID |    5 | 103030, 41489
      2672 |       16 |   ID |    6 | 144519, 40845
      2688 |       16 |   ID |    7 | 185364, 34948
      2704 |       16 |   ID |    8 | 220312, 17650
      2720 |       16 |   ID |    9 | 237962, 15024
      2736 |       16 |   ID |   10 | 252986, 44647
      2752 |       16 |   ID |   11 | 297633, 38092
      2768 |       16 |   ID |   12 | 335725, 43666
      2784 |       16 |   ID |   13 | 379391, 48523
      2800 |       16 |   ID |   14 | 427914, 46992
      2816 |       16 |   ID |   15 | 474906, 36063
      2832 |       16 |   ID |   16 | 510969, 20737
      2848 |       16 |   ID |   17 | 531706, 25756
      2864 |       16 |   ID |   18 | 557462, 58455
      2880 |       16 |   ID |   19 | 615917, 44896
      2896 |       16 |   ID |   20 | 660813, 39730
      2912 |       16 |   ID |   21 | 700543, 38596
      2928 |       16 |   ID |   22 | 739139, 31795
      2944 |       16 |   ID |   23 | 770934, 33214
      2960 |       16 |   ID |   24 | 804148, 18858
      2976 |       16 |   ID |   25 | 823006, 35122
      2992 |       16 |   ID |   26 | 858128, 41015
      3008 |       16 |   ID |   27 | 899143, 40362
      3024 |       16 |   ID |   28 | 939505, 38288
      3040 |       16 |   ID |   29 | 977793, 34497
      3056 |       16 |   ID |   30 | 1012290, 31806
      3072 |       16 |   ID |   31 | 1044096, 25360
      3088 |       16 |   ID |   32 | 1069456, 19672
      3104 |       16 |   ID |   33 | 1089128, 25400
      3120 |       16 |   ID |   34 | 1114528, 34515
      3136 |       16 |   ID |   35 | 1149043, 20863
      3152 |       16 |   ID |   36 | 1169906, 25975
      3168 |       16 |   ID |   37 | 1195881, 33532
      3184 |       16 |   ID |   38 | 1229413, 31296
      3200 |       16 |   ID |   39 | 1260709, 29869
      3216 |       16 |   ID |   40 | 1290578, 16479
      3232 |       16 |   ID |   41 | 1307057, 17323
      3248 |       16 |   ID |   42 | 1324380, 17331
      3264 |       16 |   ID |   43 | 1341711, 13357
      3280 |       16 |   ID |   44 | 1355068, 22206
      3296 |       16 |   ID |   45 | 1377274, 31859
      3312 |       16 |   ID |   46 | 1409133, 26694
      3328 |       16 |   ID |   47 | 1435827, 23620
      3344 |       16 |   ID |   48 | 1459447, 15007
      3360 |       16 |   ID |   49 |      0,     8
      3376 |       16 |   ID |   50 |   3432, 17469
      3392 |       16 |   ID |   51 |  20901,  2364
Exiv2::BMFF Exif: ID = 51 from,length = 20901,2364
Exiv2::BmffImage::boxHandler: mdat     3416->1 
""","","""data:AAACJGFwcGwEAAAAbW50clJHQiBYWVogB+EABwAHAA0AFgAgYWNzcEF
QUEwAAAAAQVBQTAAAAAAAAAAAAAAAAAAAAAAAAPbWAAEAAAAA0y1hcHBsyhq
VgiV/EE04mRPV0eoVggAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
KZGVzYwAAAPwAAABlY3BydAAAAWQAAAAjd3RwdAAAAYgAAAAUclhZWgAAAZw
AAAAUZ1hZWgAAAbAAAAAUYlhZWgAAAcQAAAAUclRSQwAAAdgAAAAgY2hhZAA
AAfgAAAAsYlRSQwAAAdgAAAAgZ1RSQwAAAdgAAAAgZGVzYwAAAAAAAAALRGl
zcGxheSBQMwAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB
0ZXh0AAAAAENvcHlyaWdodCBBcHBsZSBJbmMuLCAyMDE3AABYWVogAAAAAAA
A81EAAQAAAAEWzFhZWiAAAAAAAACD3wAAPb////+7WFlaIAAAAAAAAEq/AAC
xNwAACrlYWVogAAAAAAAAKDgAABELAADIuXBhcmEAAAAAAAMAAAACZmYAAPK
nAAANWQAAE9AAAApbc2YzMgAAAAAAAQxCAAAF3v//8yYAAAeTAAD9kP//+6L
///2jAAAD3AAAwG4=
"""]

class pr_1475_IMG_3578_heic(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/IMG_3578.heic"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2  -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS          $filename"
                   ,"$exiv2 -pX          $filename"
                   ,"$exiv2 -pC --binary $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.Make                              Ascii       6  Apple
Exif.Image.DateTime                          Ascii      20  2020:04:17 10:03:40
Exif.Photo.ExposureTime                      Rational    1  1/60 s
Exif.Photo.ExposureProgram                   Short       1  Auto
Exif.Photo.DateTimeOriginal                  Ascii      20  2020:04:17 10:03:40
Exif.Photo.DateTimeDigitized                 Ascii      20  2020:04:17 10:03:40
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ExposureMode                      Short       1  Auto
""","""Exiv2::BmffImage::boxHandler: ftyp        0->32 brand: heic
Exiv2::BmffImage::boxHandler: meta       32->3380 
  Exiv2::BmffImage::boxHandler: hdlr       44->34 
  Exiv2::BmffImage::boxHandler: dinf       78->36 
  Exiv2::BmffImage::boxHandler: pitm      114->14 
  Exiv2::BmffImage::boxHandler: iinf      128->1085 
    Exiv2::BmffImage::boxHandler: infe      142->21 ID =   1 hvc1 
    Exiv2::BmffImage::boxHandler: infe      163->21 ID =   2 hvc1 
    Exiv2::BmffImage::boxHandler: infe      184->21 ID =   3 hvc1 
    Exiv2::BmffImage::boxHandler: infe      205->21 ID =   4 hvc1 
    Exiv2::BmffImage::boxHandler: infe      226->21 ID =   5 hvc1 
    Exiv2::BmffImage::boxHandler: infe      247->21 ID =   6 hvc1 
    Exiv2::BmffImage::boxHandler: infe      268->21 ID =   7 hvc1 
    Exiv2::BmffImage::boxHandler: infe      289->21 ID =   8 hvc1 
    Exiv2::BmffImage::boxHandler: infe      310->21 ID =   9 hvc1 
    Exiv2::BmffImage::boxHandler: infe      331->21 ID =  10 hvc1 
    Exiv2::BmffImage::boxHandler: infe      352->21 ID =  11 hvc1 
    Exiv2::BmffImage::boxHandler: infe      373->21 ID =  12 hvc1 
    Exiv2::BmffImage::boxHandler: infe      394->21 ID =  13 hvc1 
    Exiv2::BmffImage::boxHandler: infe      415->21 ID =  14 hvc1 
    Exiv2::BmffImage::boxHandler: infe      436->21 ID =  15 hvc1 
    Exiv2::BmffImage::boxHandler: infe      457->21 ID =  16 hvc1 
    Exiv2::BmffImage::boxHandler: infe      478->21 ID =  17 hvc1 
    Exiv2::BmffImage::boxHandler: infe      499->21 ID =  18 hvc1 
    Exiv2::BmffImage::boxHandler: infe      520->21 ID =  19 hvc1 
    Exiv2::BmffImage::boxHandler: infe      541->21 ID =  20 hvc1 
    Exiv2::BmffImage::boxHandler: infe      562->21 ID =  21 hvc1 
    Exiv2::BmffImage::boxHandler: infe      583->21 ID =  22 hvc1 
    Exiv2::BmffImage::boxHandler: infe      604->21 ID =  23 hvc1 
    Exiv2::BmffImage::boxHandler: infe      625->21 ID =  24 hvc1 
    Exiv2::BmffImage::boxHandler: infe      646->21 ID =  25 hvc1 
    Exiv2::BmffImage::boxHandler: infe      667->21 ID =  26 hvc1 
    Exiv2::BmffImage::boxHandler: infe      688->21 ID =  27 hvc1 
    Exiv2::BmffImage::boxHandler: infe      709->21 ID =  28 hvc1 
    Exiv2::BmffImage::boxHandler: infe      730->21 ID =  29 hvc1 
    Exiv2::BmffImage::boxHandler: infe      751->21 ID =  30 hvc1 
    Exiv2::BmffImage::boxHandler: infe      772->21 ID =  31 hvc1 
    Exiv2::BmffImage::boxHandler: infe      793->21 ID =  32 hvc1 
    Exiv2::BmffImage::boxHandler: infe      814->21 ID =  33 hvc1 
    Exiv2::BmffImage::boxHandler: infe      835->21 ID =  34 hvc1 
    Exiv2::BmffImage::boxHandler: infe      856->21 ID =  35 hvc1 
    Exiv2::BmffImage::boxHandler: infe      877->21 ID =  36 hvc1 
    Exiv2::BmffImage::boxHandler: infe      898->21 ID =  37 hvc1 
    Exiv2::BmffImage::boxHandler: infe      919->21 ID =  38 hvc1 
    Exiv2::BmffImage::boxHandler: infe      940->21 ID =  39 hvc1 
    Exiv2::BmffImage::boxHandler: infe      961->21 ID =  40 hvc1 
    Exiv2::BmffImage::boxHandler: infe      982->21 ID =  41 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1003->21 ID =  42 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1024->21 ID =  43 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1045->21 ID =  44 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1066->21 ID =  45 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1087->21 ID =  46 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1108->21 ID =  47 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1129->21 ID =  48 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1150->21 ID =  49 grid 
    Exiv2::BmffImage::boxHandler: infe     1171->21 ID =  50 hvc1 
    Exiv2::BmffImage::boxHandler: infe     1192->21 ID =  51 Exif  *** Exif ***
  Exiv2::BmffImage::boxHandler: iref     1213->148 
  Exiv2::BmffImage::boxHandler: iprp     1361->1203 
    Exiv2::BmffImage::boxHandler: ipco     1369->876 
      Exiv2::BmffImage::boxHandler: colr     1377->560 
      Exiv2::BmffImage::boxHandler: hvcC     1937->112 
      Exiv2::BmffImage::boxHandler: ispe     2049->20 pixelWidth_, pixelHeight_ = 512, 512
      Exiv2::BmffImage::boxHandler: ispe     2069->20 pixelWidth_, pixelHeight_ = 4032, 3024
      Exiv2::BmffImage::boxHandler: irot     2089->9 
      Exiv2::BmffImage::boxHandler: pixi     2098->16 
      Exiv2::BmffImage::boxHandler: hvcC     2114->111 
      Exiv2::BmffImage::boxHandler: ispe     2225->20 pixelWidth_, pixelHeight_ = 320, 240
    Exiv2::BmffImage::boxHandler: ipma     2245->319 
  Exiv2::BmffImage::boxHandler: idat     2564->16 
  Exiv2::BmffImage::boxHandler: iloc     2580->832 
      2588 |       16 |   ID |    1 |  10931, 11943
      2604 |       16 |   ID |    2 |  22874, 24817
      2620 |       16 |   ID |    3 |  47691, 20110
      2636 |       16 |   ID |    4 |  67801, 19449
      2652 |       16 |   ID |    5 |  87250, 19714
      2668 |       16 |   ID |    6 | 106964, 20223
      2684 |       16 |   ID |    7 | 127187, 18124
      2700 |       16 |   ID |    8 | 145311,  8745
      2716 |       16 |   ID |    9 | 154056, 21761
      2732 |       16 |   ID |   10 | 175817, 25378
      2748 |       16 |   ID |   11 | 201195, 16664
      2764 |       16 |   ID |   12 | 217859, 18638
      2780 |       16 |   ID |   13 | 236497, 22750
      2796 |       16 |   ID |   14 | 259247, 15801
      2812 |       16 |   ID |   15 | 275048, 15063
      2828 |       16 |   ID |   16 | 290111, 15671
      2844 |       16 |   ID |   17 | 305782, 19360
      2860 |       16 |   ID |   18 | 325142, 19415
      2876 |       16 |   ID |   19 | 344557, 29895
      2892 |       16 |   ID |   20 | 374452, 21193
      2908 |       16 |   ID |   21 | 395645, 46228
      2924 |       16 |   ID |   22 | 441873,  9356
      2940 |       16 |   ID |   23 | 451229, 14175
      2956 |       16 |   ID |   24 | 465404, 16734
      2972 |       16 |   ID |   25 | 482138, 32304
      2988 |       16 |   ID |   26 | 514442, 44251
      3004 |       16 |   ID |   27 | 558693, 21635
      3020 |       16 |   ID |   28 | 580328, 27907
      3036 |       16 |   ID |   29 | 608235,  8462
      3052 |       16 |   ID |   30 | 616697, 17997
      3068 |       16 |   ID |   31 | 634694, 22077
      3084 |       16 |   ID |   32 | 656771, 17266
      3100 |       16 |   ID |   33 | 674037, 19623
      3116 |       16 |   ID |   34 | 693660, 22028
      3132 |       16 |   ID |   35 | 715688, 22078
      3148 |       16 |   ID |   36 | 737766, 21911
      3164 |       16 |   ID |   37 | 759677, 20937
      3180 |       16 |   ID |   38 | 780614, 35413
      3196 |       16 |   ID |   39 | 816027, 53869
      3212 |       16 |   ID |   40 | 869896, 33923
      3228 |       16 |   ID |   41 | 903819, 18130
      3244 |       16 |   ID |   42 | 921949, 16268
      3260 |       16 |   ID |   43 | 938217, 23620
      3276 |       16 |   ID |   44 | 961837, 31499
      3292 |       16 |   ID |   45 | 993336, 33513
      3308 |       16 |   ID |   46 | 1026849, 37642
      3324 |       16 |   ID |   47 | 1064491, 37734
      3340 |       16 |   ID |   48 | 1102225, 20085
      3356 |       16 |   ID |   49 |      0,     8
      3372 |       16 |   ID |   50 |   3428,  5479
      3388 |       16 |   ID |   51 |   8907,  2024
Exiv2::BMFF Exif: ID = 51 from,length = 8907,2024
Exiv2::BmffImage::boxHandler: mdat     3412->1 
""","","""data:AAACJGFwcGwEAAAAbW50clJHQiBYWVogB+EABwAHAA0AFgAgYWNzcEF
QUEwAAAAAQVBQTAAAAAAAAAAAAAAAAAAAAAAAAPbWAAEAAAAA0y1hcHBsyhq
VgiV/EE04mRPV0eoVggAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
KZGVzYwAAAPwAAABlY3BydAAAAWQAAAAjd3RwdAAAAYgAAAAUclhZWgAAAZw
AAAAUZ1hZWgAAAbAAAAAUYlhZWgAAAcQAAAAUclRSQwAAAdgAAAAgY2hhZAA
AAfgAAAAsYlRSQwAAAdgAAAAgZ1RSQwAAAdgAAAAgZGVzYwAAAAAAAAALRGl
zcGxheSBQMwAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB
0ZXh0AAAAAENvcHlyaWdodCBBcHBsZSBJbmMuLCAyMDE3AABYWVogAAAAAAA
A81EAAQAAAAEWzFhZWiAAAAAAAACD3wAAPb////+7WFlaIAAAAAAAAEq/AAC
xNwAACrlYWVogAAAAAAAAKDgAABELAADIuXBhcmEAAAAAAAMAAAACZmYAAPK
nAAANWQAAE9AAAApbc2YzMgAAAAAAAQxCAAAF3v//8yYAAAeTAAD9kP//+6L
///2jAAAD3AAAwG4=
"""]

class pr_1475_Stonehenge_heic(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/Stonehenge.heic"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 -g Image.Make -g Date -g Xm -g Expo -g Flash $filename"
                   ,"$exiv2 -pS          $filename"
                   ,"$exiv2 -pX          $filename"
                   ,"$exiv2 -pC --binary $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["""Exif.Image.Make                              Ascii      18  NIKON CORPORATION
Exif.Image.DateTime                          Ascii      20  2015:07:16 20:25:28
Exif.Photo.ExposureTime                      Rational    1  1/400 s
Exif.Photo.ExposureProgram                   Short       1  Not defined
Exif.Photo.DateTimeOriginal                  Ascii      20  2015:07:16 15:38:54
Exif.Photo.DateTimeDigitized                 Ascii      20  2015:07:16 15:38:54
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Photo.FlashpixVersion                   Undefined   4  1.00
Exif.Photo.ExposureMode                      Short       1  Auto
Xmp.aux.SerialNumber                         XmpText     7  2567806
Xmp.aux.FlashCompensation                    XmpText     3  0/1
Xmp.aux.LensInfo                             XmpText    20  18/1 250/1 7/2 63/10
Xmp.aux.Lens                                 XmpText    23  18.0-250.0 mm f/3.5-6.3
Xmp.aux.ImageNumber                          XmpText     4  9608
Xmp.aux.LensID                               XmpText    20  -7910804157571773682
Xmp.xmp.Rating                               XmpText     1  0
Xmp.xmp.CreatorTool                          XmpText     9  Ver.1.00 
Xmp.xmp.CreateDate                           XmpText    22  2015-07-16T15:38:54.00
Xmp.xmp.ModifyDate                           XmpText    28  2015-07-16T20:25:28.00+01:00
Xmp.cm2e.Father                              XmpText    11  Robin Mills
Xmp.cm2e.Family                              XmpBag      0  
Xmp.photoshop.DateCreated                    XmpText    22  2015-07-16T15:38:54.00
Xmp.dc.description                           LangAlt     1  lang="x-default" Classic View
Xmp.dc.Family                                XmpBag      1  Robin
""","""Exiv2::BmffImage::boxHandler: ftyp        0->24 brand: heic
Exiv2::BmffImage::boxHandler: meta       24->508 
  Exiv2::BmffImage::boxHandler: hdlr       36->34 
  Exiv2::BmffImage::boxHandler: dinf       70->36 
  Exiv2::BmffImage::boxHandler: pitm      106->14 
  Exiv2::BmffImage::boxHandler: iinf      120->97 
    Exiv2::BmffImage::boxHandler: infe      134->21 ID =   1 hvc1 
    Exiv2::BmffImage::boxHandler: infe      155->21 ID =   2 Exif  *** Exif ***
    Exiv2::BmffImage::boxHandler: infe      176->41 ID =   3 mime  *** XMP ***
  Exiv2::BmffImage::boxHandler: iref      217->40 
  Exiv2::BmffImage::boxHandler: iprp      257->217 
    Exiv2::BmffImage::boxHandler: ipco      265->185 
      Exiv2::BmffImage::boxHandler: colr      273->19 
      Exiv2::BmffImage::boxHandler: hvcC      292->113 
      Exiv2::BmffImage::boxHandler: ispe      405->20 pixelWidth_, pixelHeight_ = 300, 200
      Exiv2::BmffImage::boxHandler: irot      425->9 
      Exiv2::BmffImage::boxHandler: pixi      434->16 
    Exiv2::BmffImage::boxHandler: ipma      450->24 
  Exiv2::BmffImage::boxHandler: iloc      474->58 
       482 |       14 |   ID |    1 |   4780, 13461
       496 |       14 |   ID |    2 |    548,  1082
       510 |       14 |   ID |    3 |   1630,  3150
Exiv2::BMFF Exif: ID = 2 from,length = 548,1082
Exiv2::BMFF XMP: ID = 3 from,length = 1630,3150
Exiv2::BmffImage::boxHandler: mdat      532->1 
""","""<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:aux="http://ns.adobe.com/exif/1.0/aux/"
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
    xmlns:cm2e="http://clanmills.com/exiv2/"
    xmlns:photoshop="http://ns.adobe.com/photoshop/1.0/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
   aux:SerialNumber="2567806"
   aux:FlashCompensation="0/1"
   aux:LensInfo="18/1 250/1 7/2 63/10"
   aux:Lens="18.0-250.0 mm f/3.5-6.3"
   aux:ImageNumber="9608"
   aux:LensID="-7910804157571773682"
   xmp:Rating="0"
   xmp:CreatorTool="Ver.1.00 "
   xmp:CreateDate="2015-07-16T15:38:54.00"
   xmp:ModifyDate="2015-07-16T20:25:28.00+01:00"
   cm2e:Father="Robin Mills"
   photoshop:DateCreated="2015-07-16T15:38:54.00">
   <cm2e:Family>
    <rdf:Bag/>
   </cm2e:Family>
   <dc:description>
    <rdf:Alt>
     <rdf:li xml:lang="x-default">Classic View</rdf:li>
    </rdf:Alt>
   </dc:description>
   <dc:Family>
    <rdf:Bag>
     <rdf:li>Robin</rdf:li>
    </rdf:Bag>
   </dc:Family>
  </rdf:Description>
 </rdf:RDF>
</x:xmpmeta>
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                                                                                                    
                           
<?xpacket end="w"?>""",""]

class pr_1475_heic_heic(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/1475"
    filename = "$data_path/heic.heic"
    if bSkip:
        commands=[]
        retval=[]
        stdin=[]
        stderr=[]
        stdout=[]
        print("*** test skipped.  requires enable_bmff=1***")
    else:
        commands = ["$exiv2 -pa $filename"
                   ,"$exiv2 -pS          $filename"
                   ,"$exiv2 -pX          $filename"
                   ,"$exiv2 -pC --binary $filename"
                   ]
        retval = [ 0  ] * len(commands)
        stderr = [ "" ] * len(commands)
        stdin  = [ "" ] * len(commands)
        stdout = ["","""Exiv2::BmffImage::boxHandler: ftyp        0->28 brand: mif1
Exiv2::BmffImage::boxHandler: meta       28->921 
  Exiv2::BmffImage::boxHandler: hdlr       40->33 
  Exiv2::BmffImage::boxHandler: pitm       73->14 
  Exiv2::BmffImage::boxHandler: iloc       87->88 
        95 |       18 |   ID | 20004 |    957,333704
       113 |       18 |   ID | 20005 | 334669, 24523
       131 |       18 |   ID | 20006 | 359200,330132
       149 |       18 |   ID | 20007 | 689340, 28758
  Exiv2::BmffImage::boxHandler: iinf      175->140 
  Exiv2::BmffImage::boxHandler: iref      315->40 
  Exiv2::BmffImage::boxHandler: iprp      355->594 
    Exiv2::BmffImage::boxHandler: ipco      363->550 
      Exiv2::BmffImage::boxHandler: hvcC      371->126 
      Exiv2::BmffImage::boxHandler: ispe      497->20 pixelWidth_, pixelHeight_ = 1280, 854
      Exiv2::BmffImage::boxHandler: hvcC      517->125 
      Exiv2::BmffImage::boxHandler: ispe      642->20 pixelWidth_, pixelHeight_ = 320, 212
      Exiv2::BmffImage::boxHandler: hvcC      662->126 
      Exiv2::BmffImage::boxHandler: hvcC      788->125 
    Exiv2::BmffImage::boxHandler: ipma      913->36 
Exiv2::BmffImage::boxHandler: mdat      949->333712 
Exiv2::BmffImage::boxHandler: mdat   334661->24531 
Exiv2::BmffImage::boxHandler: mdat   359192->330140 
Exiv2::BmffImage::boxHandler: mdat   689332->28766 
Exiv2::BmffImage::boxHandler: mdat   718098->16 
""","",""]




# -*- coding: utf-8 -*-

import system_tests

class CheckTiffPageNumber(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1044"

    filename = system_tests.path("$data_path/exiv2-bug1044.tif")
    commands = [ "$exiv2 -q -pa -g PageNumber $filename" ]

    stdout = [ """Exif.Image.PageNumber                        Short       2  0 1
"""
    ]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")

class test_issue_547(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/pull/547
    """
    url = "https://github.com/Exiv2/exiv2/issues/547"

    filename = path("$data_path/issue_547.poc")
    commands = ["$exiv2 -pR $filename"]
    stdout = ["""STRUCTURE OF TIFF FILE (II): $filename
 address |    tag                              |      type |    count |    offset | value
      12 | 0x0001 Version                      |    SSHORT |        0 |           | 
"""]
    stderr = [
"""Exiv2 exception in print action for file $filename:
$kerCorruptedMetadata
"""
]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class OutputTagExtract(metaclass=system_tests.CaseMeta):
    """
    Test whether exiv2 -pa $file and exiv2 -pS $file produces the same output.
    """

    def parse_pa(self, stdout):
        """
        Parse the output of exiv2 -pa $file, which looks like this:

        Exif.Image.NewSubfileType                    Long        1  Primary image

        into a list of dictionaries with the keys:
        tag: last word of the first column (here NewSubfileType)
        type: lowercase second column
        len: third column
        val: fourth column

        It is furthermore checked that the first column begins with 'Exif.Image'
        """
        data = []

        for line in stdout:
            tmp = line.split()

            exif, image, tag = tmp[0].split('.')
            self.assertEquals(exif, "Exif")
            self.assertEquals(image, "Image")

            data.append({
                "tag": tag,
                "type": tmp[1].lower(),
                "len": int(tmp[2]),
                "val": " ".join(tmp[3:])
            })

        return data

    def parse_pS(self, stdout):
        """
        Parse the output of exiv2 -pS $file, which looks like this:

        STRUCTURE OF TIFF FILE (II): $file
        address |    tag                              |      type |    count |    offset | value
        254 | 0x00fe NewSubfileType               |      LONG |        1 |           | 0
         ...
        END $file

        into a list of dictionaries with the following keys:
        tag: the string after the hex number in the second column
        type: lowercase third column
        len: fourth column
        val: fifth column

        The first two lines and the last line are ignored, as they contain
        explanatory output.
        """
        data = []

        for i, line in enumerate(stdout):
            if i < 2 or i == len(stdout) - 1:
                continue

            tmp = line.split(" | ")
            data.append({
                "tag": tmp[1].split()[1],
                "type": tmp[2].replace(' ', '').lower(),
                "len": int(tmp[3].replace(' ', '')),
                "val": tmp[5]
            })

        return data

    def compare_pS_pa(self):
        """
        Compares the output from self.parse_pa() and self.parse_pS() (saved in
        self.pa_data & self.pS_data respectively).
        All dictionaries in the lists are compared for equality for the keys
        tag, len and type but only some for val. This is due to differently
        processed output (exiv2 -pa produces more readable output,
        e.g. compression is written in words and not as a number as it is by
        exiv2 -pS)
        """
        for pa_elem, pS_elem in zip(self.pa_data, self.pS_data):
            for key in ["tag", "type", "len"]:
                self.assertEquals(pa_elem[key], pS_elem[key])

            if pa_elem["tag"] in [
                    "ImageWidth", "ImageLength", "BitsPerSample",
                    "DocumentName", "ImageDescription", "StripOffsets",
                    "SamplesPerPixel", "StripByteCounts"]:
                self.assertEquals(pa_elem["val"], pS_elem["val"])

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        super().compare_stdout(i, command, got_stdout, expected_stdout)

        if '-pa' in command:
            self.pa_data = self.parse_pa(got_stdout.splitlines())
        if '-pS' in command:
            self.pS_data = self.parse_pS(got_stdout.splitlines())

        if i == 1:
            self.compare_pS_pa()

    commands = [
        "$exiv2 %s $data_path/mini9.tif" % (opt) for opt in ["-pa", "-pS"]
    ]

    stderr = [""] * 2
    retval = [0] * 2
    stdout = [
        """Exif.Image.NewSubfileType                    Long        1  Primary image
Exif.Image.ImageWidth                        Short       1  9
Exif.Image.ImageLength                       Short       1  9
Exif.Image.BitsPerSample                     Short       3  8 8 8
Exif.Image.Compression                       Short       1  Uncompressed
Exif.Image.PhotometricInterpretation         Short       1  RGB
Exif.Image.DocumentName                      Ascii      24  /home/ahuggel/mini9.tif
Exif.Image.ImageDescription                  Ascii      18  Created with GIMP
Exif.Image.StripOffsets                      Long        1  8
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.SamplesPerPixel                   Short       1  3
Exif.Image.RowsPerStrip                      Short       1  64
Exif.Image.StripByteCounts                   Long        1  243
Exif.Image.XResolution                       Rational    1  72
Exif.Image.YResolution                       Rational    1  72
Exif.Image.PlanarConfiguration               Short       1  Chunky
Exif.Image.ResolutionUnit                    Short       1  inch
""",
        """STRUCTURE OF TIFF FILE (II): $data_path/mini9.tif
 address |    tag                              |      type |    count |    offset | value
     254 | 0x00fe NewSubfileType               |      LONG |        1 |           | 0
     266 | 0x0100 ImageWidth                   |     SHORT |        1 |           | 9
     278 | 0x0101 ImageLength                  |     SHORT |        1 |           | 9
     290 | 0x0102 BitsPerSample                |     SHORT |        3 |       462 | 8 8 8
     302 | 0x0103 Compression                  |     SHORT |        1 |           | 1
     314 | 0x0106 PhotometricInterpretation    |     SHORT |        1 |           | 2
     326 | 0x010d DocumentName                 |     ASCII |       24 |       468 | /home/ahuggel/mini9.tif
     338 | 0x010e ImageDescription             |     ASCII |       18 |       492 | Created with GIMP
     350 | 0x0111 StripOffsets                 |      LONG |        1 |           | 8
     362 | 0x0112 Orientation                  |     SHORT |        1 |           | 1
     374 | 0x0115 SamplesPerPixel              |     SHORT |        1 |           | 3
     386 | 0x0116 RowsPerStrip                 |     SHORT |        1 |           | 64
     398 | 0x0117 StripByteCounts              |      LONG |        1 |           | 243
     410 | 0x011a XResolution                  |  RATIONAL |        1 |       510 | 1207959552/16777216
     422 | 0x011b YResolution                  |  RATIONAL |        1 |       518 | 1207959552/16777216
     434 | 0x011c PlanarConfiguration          |     SHORT |        1 |           | 1
     446 | 0x0128 ResolutionUnit               |     SHORT |        1 |           | 2
END $data_path/mini9.tif
"""]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class DontSegfaultWhenAccessingFocalLength(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/855"

    filename = path("$data_path/exiv2-bug855.jpg")

    commands = ["$exiv2 -pa -b $filename"]

    stdout = ["""Exif.Image.NewSubfileType                    Long        1  Thumbnail/Preview image
Exif.Image.Make                              Ascii      20  PENTAX             
Exif.Image.Model                             Ascii      20  PENTAX K-x         
Exif.Image.Orientation                       Short       1  top, left
Exif.Image.Software                          Ascii      16  darktable 0.9.3
Exif.Image.DateTime                          Ascii      20  2012:01:22 01:54:26
Exif.Image.Artist                            Ascii      17  SEBASTIAN WAGNER
Exif.Image.Rating                            SLong       1  1
Exif.Image.RatingPercent                     SLong       1  20
Exif.Image.Copyright                         Ascii      33  CREATIVECOMMONS ATTRIBUTION CCBY
Exif.Image.ExifTag                           Long        1  666
Exif.Photo.ExposureTime                      Rational    1  1/20 s
Exif.Photo.FNumber                           Rational    1  F5.6
Exif.Photo.ExposureProgram                   Short       1  Landscape mode
Exif.Photo.ISOSpeedRatings                   Short       1  2500
Exif.Photo.DateTimeOriginal                  Ascii      20  2012:01:22 01:54:26
Exif.Photo.DateTimeDigitized                 Ascii      20  2012:01:22 01:54:26
Exif.Photo.ExposureBiasValue                 SRational   1  0 EV
Exif.Photo.MeteringMode                      Short       1  Multi-segment
Exif.Photo.Flash                             Short       1  No, compulsory
Exif.Photo.FocalLength                       Rational    0  
Exif.Photo.ColorSpace                        Short       1  sRGB
Exif.Photo.SensingMethod                     Short       1  One-chip color area
Exif.Photo.CustomRendered                    Short       1  Normal process
Exif.Photo.ExposureMode                      Short       1  Auto
Exif.Photo.WhiteBalance                      Short       1  Auto
Exif.Photo.FocalLengthIn35mmFilm             Short       1  82.0 mm
Exif.Photo.SceneCaptureType                  Short       1  Landscape
Exif.Photo.Contrast                          Short       1  Hard
Exif.Photo.Saturation                        Short       1  Normal
Exif.Photo.Sharpness                         Short       1  Hard
Exif.Photo.SubjectDistanceRange              Short       1  Macro
Exif.Image.DNGVersion                        Byte        4  1 1 0 0
Exif.Image.DNGBackwardVersion                Byte        4  1 1 0 0
Exif.Image.UniqueCameraModel                 Ascii      11  PENTAX K-x
Exif.Image.ColorMatrix1                      SRational   9  71147/65536 -36788/65536 223/65536 -28867/65536 69463/65536 28977/65536 -1276/65536 2064/65536 45215/65536
Exif.Image.ColorMatrix2                      SRational   9  68429/65536 -21793/65536 -7522/65536 -36512/65536 89125/65536 14083/65536 -7905/65536 11496/65536 48817/65536
Exif.Image.AnalogBalance                     Rational    3  1/1 1/1 1/1
Exif.Image.AsShotNeutral                     Rational    3  256/352 256/264 256/762
Exif.Image.BaselineExposure                  SRational   1  -31775/65536
Exif.Image.BaselineNoise                     Rational    1  1/1
Exif.Image.BaselineSharpness                 Rational    1  1/1
Exif.Image.LinearResponseLimit               Rational    1  1/1
Exif.Image.CalibrationIlluminant1            Short       1  Standard light A
Exif.Image.CalibrationIlluminant2            Short       1  D65
"""
    ]
    stderr = [
        """Error: Upper boundary of data for directory Photo, entry 0x920a is out of bounds: Offset = 0x000003dc, size = 8, exceeds buffer size by 6 Bytes; truncating the entry
"""
    ]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")
class SubBoxLengthDataBufAbort(metaclass=system_tests.CaseMeta):

    url = "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2018-9145"

    filename = system_tests.path(
        "$data_path/4-DataBuf-abort-1"
    )
    commands = ["$exiv2 -pR $filename"]
    stdout = [
        """STRUCTURE OF JPEG2000 FILE: $filename
 address |   length | box       | data
       0 |       12 | jP        | 
      12 |       20 | jp2h      | 
"""
    ]
    stderr = [
        """$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class LargeAllocationInPgfReadMetadata(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/847

    An unchecked allocation size causes a std::bad_alloc to
    be thrown.
    """
    url = "https://github.com/Exiv2/exiv2/issues/847"

    filename = path("$data_path/issue_847_poc.pgf")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
Failed to read input data
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-bug480.jpg")
class LargeIptcTest(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/480"

    bug_file = "$data_path/exiv2-bug480_copy.jpg"
    imagemagick_file = "$data_path/imagemagick.png"

    commands = ["$largeiptc_test $bug_file $imagemagick_file"]

    stdout = ["""Reading 144766 bytes from $imagemagick_file
IPTC fields: 0
IPTC fields: 144775
IRB buffer : 144788
Comparing IPTC and IRB size... ok
Comparing IPTC and IRB data... ok
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path


class Canon_EF_35mm_f_slash_2_IS_USM_Lens(metaclass=CaseMeta):

    url = "http://dev.exiv2.org/issues/876"

    filename = path("$data_path/exiv2-bug876.jpg")

    commands = ["$exiv2 -pt --grep Model $filename"]

    stdout = ["""Exif.Image.Model                             Ascii      13  Canon EOS 6D
Exif.Canon.ModelID                           Long        1  EOS 6D
Exif.Canon.LensModel                         Ascii      74  EF35mm f/2 IS USM
"""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path

class test_issue_ghsa_mv9g_fxh2_m49m(metaclass=CaseMeta):

    filename = path("$data_path/test_issue_ghsa_mv9g_fxh2_m49m.crw")
    commands  = ["$exiv2 -q fi $filename"]
    stdout = [""]
    stderr = ["""Exiv2 exception in fixiso action for file $filename:
$kerCorruptedMetadata
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class NikonSigmaLens_APO_MACRO_180_F35_EX_DG_HSM(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/pull/992"
    
    filename = "$data_path/Sigma_APO_MACRO_180_F3.5_EX_DG_HSM.exv"
    commands = ["$exiv2 -pa --grep lensid/i $filename"]
    stderr = [""]
    stdout = [""
        """Exif.NikonLd3.LensIDNumber                   Byte        1  Sigma APO Macro 180mm F3.5 EX DG HSM
"""
]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class BufferOverflowInSscanf(metaclass=system_tests.CaseMeta):
    """
    This requires some sort of memchecker, either valgrind or ASAN.
    """

    url = "http://dev.exiv2.org/issues/447"

    filename = "$data_path/exiv2-bug447.jpg"
    commands = ["$exiv2 -u -pi $filename"]

    stdout = ["""Iptc.Application2.Caption                    String      0  
Iptc.Application2.DateCreated                Date        8  2005-08-09
Iptc.Application2.TimeCreated                Time       11  01:28:31-07:00
"""]
    stderr = ["""Error: Directory NikonPreview with 8224 entries considered invalid; not read.
Warning: Directory Photo, entry 0x9286 has unknown Exif (TIFF) type 14; setting type size 1.
"""]
    retval = [0]



# -*- coding: utf-8 -*-

import system_tests


class TamronSupport(metaclass=system_tests.CaseMeta):

    description = "Added support for 'Tamron SP 15-30mm f/2.8 Di VC USD' and 'Tamron SP 90mm f/2.8 Di VC USD MACRO1:1'"

    files = [
        "exiv2-g20.exv",
        "CanonEF100mmF2.8LMacroISUSM.exv",
        "TamronSP15-30mmF2.8DiVCUSDA012.exv",
        "TamronSP90mmF2.8DiVCUSDMacroF004.exv",
        "TamronSP90mmF2.8DiVCUSDMacroF017.exv"
    ]
    commands = ["$exiv2 -pa --grep lens/i ../../../test/data/" + files[0]] \
        + list(map(
            lambda fname: "$exiv2 -pa --grep lenstype/i ../../../test/data/" + fname,
            files[1:]
        ))
    retval = [0] * len(files)

    stdout = [
        # exiv2-g20.exv
        """Exif.CanonCs.LensType                        Short       1  Tamron SP 90mm f/2.8 Di VC USD Macro 1:1
Exif.CanonCs.Lens                            Short       3  90.0 mm
Exif.Canon.LensModel                         Ascii      70  TAMRON SP 90mm F/2.8 Di VC USD MACRO1:1 F017
Exif.Photo.LensSpecification                 Rational    4  90/1 90/1 0/0 0/0
Exif.Photo.LensModel                         Ascii      70  TAMRON SP 90mm F/2.8 Di VC USD MACRO1:1 F017
Exif.Photo.LensSerialNumber                  Ascii      12  0000000000
""",

        # CanonEF100mmF2.8LMacroISUSM.exv
        """Exif.CanonCs.LensType                        Short       1  Canon EF 100mm f/2.8L Macro IS USM
""",

        # TamronSP15-30mmF2.8DiVCUSDA012.exv"
        """Exif.CanonCs.LensType                        Short       1  Tamron SP 15-30mm f/2.8 Di VC USD
""",

        # TamronSP90mmF2.8DiVCUSDMacroF004.exv
        """Exif.CanonCs.LensType                        Short       1  Tamron SP 90mm f/2.8 Di VC USD Macro 1:1
""",

        # TamronSP90mmF2.8DiVCUSDMacroF017.exv
        """Exif.CanonCs.LensType                        Short       1  Tamron SP 90mm f/2.8 Di VC USD Macro 1:1
"""
        ""]
    stderr = [""] * len(files)



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class AssertionSvEqualsDFailedRegression(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/769"

    filename = "$data_path/exiv2-empty_copy.jpg"

    commands = [
        """$exiv2 -u -v -M"add Exif.Image.Make Canon" -M"add Exif.CanonCs.0x0001 Short 1" -M"add Exif.CanonCs.0x0000 Short 2" $filename""",
        "$exiv2 -u -v -PEkyct $filename"
    ]

    stdout = [
        """File 1/1: $filename
Add Exif.Image.Make "Canon" (Ascii)
Add Exif.CanonCs.0x0001 "1" (Short)
Add Exif.CanonCs.0x0000 "2" (Short)
""",
        """File 1/1: $filename
Exif.Image.Make                              Ascii       6  Canon
Exif.Image.ExifTag                           Long        1  44
Exif.Photo.MakerNote                         Undefined  18  1 0 1 0 3 0 2 0 0 0 4 0 1 0 0 0 0 0
Exif.MakerNote.Offset                        Long        1  62
Exif.MakerNote.ByteOrder                     Ascii       3  II
Exif.CanonCs.0x0000                          Short       1  4
Exif.CanonCs.Macro                           Short       1  On
"""
    ]

    stderr = [""] * 2
    retval = [0] * 2



from conans import ConanFile
from conans.tools import os_info
from conans.model.version import Version

class Exiv2Conan(ConanFile):
    settings = 'os', 'compiler', 'build_type', 'arch'
    generators = 'cmake'
    options = {'unitTests': [True, False],
               'xmp': [True, False],
               'iconv': [True, False],
               'webready': [True, False],
              }
    default_options = ('unitTests=True',
                       'xmp=False',
                       'iconv=False',
                       'webready=False',
                      )

    def configure(self):
        self.options['libcurl'].shared = False
        self.options['libcurl'].with_openssl = True
        self.options['gtest'].shared = True

    def requirements(self):
        self.requires('zlib/1.2.11@conan/stable')

        if os_info.is_windows and self.options.iconv:
            self.requires('libiconv/1.15@bincrafters/stable')

        if self.options.unitTests:
            if self.settings.compiler == "Visual Studio" and Version(self.settings.compiler.version.value) <= "12":
                self.requires('gtest/1.8.0@bincrafters/stable')
            else:
                self.requires('gtest/1.8.1@bincrafters/stable')

        if self.options.webready and not os_info.is_macos:
            # Note: This difference in versions is just due to a combination of corner cases in the
            # recipes and the OS & compiler versions used in Travis and AppVeyor. In normal cases we
            # could use any of the versions.Also note that the issue was not with libcurl but with
            # libopenssl (a transitive dependency)
            if os_info.is_windows:
                self.requires('libcurl/7.69.1')
                self.options['libcurl'].with_openssl = False
                self.options['libcurl'].with_winssl = True
            else:
                self.requires('libcurl/7.64.1@bincrafters/stable')

        if self.options.xmp:
            self.requires('XmpSdk/2016.7@piponazo/stable') # from conan-piponazo
        else:
            self.requires('Expat/2.2.6@pix4d/stable')

    def imports(self):
        self.copy('*.dll', dst='conanDlls', src='bin')
        self.copy('*.dylib', dst='bin', src='lib')



# -*- coding: utf-8 -*-

import system_tests


class WebpDecodeChunksOutOfBoundsRead(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/378"

    filename = "$data_path/issue_378_1-poc-heapoverflow"

    commands = ["$exiv2 -pp $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerCorruptedMetadata
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


class DetectsWrongLengthOfImageResourceInPSDFile(metaclass=system_tests.CaseMeta):
    url = "https://github.com/Exiv2/exiv2/issues/426"

    filename = system_tests.path("$data_path/h02.psd")
    commands = ["$exiv2 " + filename]
    retval = [1]
    stderr = ["""$exiv2_exception_message """ + filename + """:
$kerCorruptedMetadata
"""]



# -*- coding: utf-8 -*-

import system_tests


URL = "https://github.com/Exiv2/exiv2/issues/590"


@system_tests.CopyFiles("$data_path/tiffimage_int-out-of-bound-read-poc-2")
class TiffImageIntOutOfBoundsRead(metaclass=system_tests.CaseMeta):
    """
    Regression test for the second issue reported in #590.
    """

    filename = system_tests.path(
        "$data_path/tiffimage_int-out-of-bound-read-poc-2_copy"
    )

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors

    commands = ["$exiv2 -Y 2011 -O 02 -D 29 adjust $filename"]
    stdout = [""]
    stderr = [""]
    retval = [0]



# -*- coding: utf-8 -*-

import hashlib

import system_tests


class IccProfileInApp2Segment(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1074"
    num = 1074

    encodings = [bytes]

    filenames = [
        system_tests.path("$data_path/" + fname)
        for fname in (
            "exiv2-bug$num.png", "imagemagick.png", "Reagan.tiff", "Reagan.jpg"
        )
    ]

    commands = [
        "$exiv2 -pC " + fname for fname in filenames
    ]

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        self.assertEqual(
            hashlib.md5(got_stdout).hexdigest(), expected_stdout
        )

    stderr = [bytes()] * len(filenames)
    stdout = [
        "5c02432934195866147d8cbfa49f3fcf",
        "cf0aeee7fdc11b20ad8a19d65628488e",
        "1d3fda2edb4a89ab60a23c5f7c7d81dd",
        "50b9125494306a6fc1b7c4f2a1a8d49d"
    ]
    retval = [0] * len(filenames)



# -*- coding: utf-8 -*-

import system_tests


class parseTXTChunkOutOfBoundsRead(metaclass=system_tests.CaseMeta):

    url = "https://github.com/Exiv2/exiv2/issues/400"

    filenames = [
        system_tests.path("$data_path/issue_400_poc" + str(i)) for i in (1, 2)
    ]

    commands = ["$exiv2 " + fname for fname in filenames]
    stdout = [""] * 2
    stderr = [
        """$exiv2_exception_message """ + fname + """:
$kerCorruptedMetadata
"""
        for fname in filenames
    ]
    retval = [1] * 2



# -*- coding: utf-8 -*-

import system_tests


class AdditionOverflowInLoaderExifJpeg(metaclass=system_tests.CaseMeta):
    """
    Regression test for bug #365:
    https://github.com/Exiv2/exiv2/issues/365
    aka CVE 2018-12265:
    https://cve.mitre.org/cgi-bin/cvename.cgi?name=2018-12265
    """
    filename = system_tests.path("$data_path/1-out-of-read-Poc")
    commands = ["$exiv2 -ep $filename"]
    stdout = [""]
    stderr = [
        """Error: Upper boundary of data for directory Image, entry 0x00fe is out of bounds: Offset = 0x0000002a, size = 64, exceeds buffer size by 22 Bytes; truncating the entry
Warning: Directory Image, entry 0x0201: Strip 0 is outside of the data area; ignored.
Warning: Directory Image, entry 0x0201: Strip 7 is outside of the data area; ignored.
Error: Offset of directory Thumbnail, entry 0x0201 is out of bounds: Offset = 0x00000000; truncating the entry
$uncaught_exception $addition_overflow_message
"""
    ]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests


@system_tests.CopyFiles("$data_path/exiv2-empty.jpg")
class SettingExifImageDateTimeDateGivesValueTooLarge(
        metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/554"

    filename = "$data_path/exiv2-empty_copy.jpg"
    commands = [
        """$exiv2 -u -v -M"set Exif.Image.DateTime Date 2007-05-27" $filename""",
        "$exiv2 -u -pt $filename"
    ]
    stdout = [
        """File 1/1: $filename
Set Exif.Image.DateTime "2007-05-27" (Date)
""",
        """Exif.Image.DateTime                          Undefined   8  50 48 48 55 48 53 50 55
"""
    ]
    stderr = [
        """Error: 'Date' is not a valid Exif (TIFF) type; using type 'Undefined'.
""",
        ""
    ]
    retval = [0] * 2



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path
@CopyTmpFiles("$data_path/Stonehenge.exv")

class test_issue_1484(metaclass=CaseMeta):
    url       = "https://github.com/Exiv2/exiv2/issues/1484"
    filename  = path("$tmp_path/Stonehenge.exv")
    commands  = [ "$exiv2 -g Copyright                            $filename" 
                , "$exiv2 -M\"set Exif.Image.Copyright Ascii ''\" $filename"
                , "$exiv2 -g Copyright                            $filename"
                , "$exiv2 -M\"del Exif.Image.Copyright Ascii\"    $filename"
                , "$exiv2 -g Copyright                            $filename"
                , "$exiv2 -M\"set Exif.Image.Copyright\"          $filename"
                , "$exiv2 -g Copyright                            $filename"
                , "$exiv2 -M\"set Exif.Image.Copyright me 2021-\" $filename"
                , "$exiv2 -g Copyright                            $filename"
                ]
    stdout   =  ["","",
"""Exif.Image.Copyright                         Ascii       1  
""","","","",
"""Exif.Image.Copyright                         Ascii       1  
""","",
"""Exif.Image.Copyright                         Ascii       9  me 2021-
"""]
    stderr = [""]*len(commands)
    retval = [1,0,0,0,1,0,0,0,0]    
    


# -*- coding: utf-8 -*-

import system_tests

class FujiTags(metaclass=system_tests.CaseMeta):

    filesAndExpectedOutput = [
("FujiTagsDRangeAutoRating1.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Rating                         Long        1  1
Exif.Fujifilm.DRangePriority                 Short       1  Auto
Exif.Fujifilm.DRangePriorityAuto             Short       1  Weak
""")
,
("FujiTagsDRangeWeakRating2.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Rating                         Long        1  2
Exif.Fujifilm.DRangePriority                 Short       1  Fixed
Exif.Fujifilm.DRangePriorityFixed            Short       1  Weak
""")
,
("FujiTagsDRangeStrongRating3.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Rating                         Long        1  3
Exif.Fujifilm.DRangePriority                 Short       1  Fixed
Exif.Fujifilm.DRangePriorityFixed            Short       1  Strong
"""),
("FujiTagsSTone0HTone0Rating4.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  0
Exif.Fujifilm.HighlightTone                  SLong       1  0
Exif.Fujifilm.Rating                         Long        1  4
"""),
("FujiTagsSTone1HTone-1Rating5.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  +1
Exif.Fujifilm.HighlightTone                  SLong       1  -1
Exif.Fujifilm.Rating                         Long        1  5
"""),
("FujiTagsSTone4HTone-2.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  +4
Exif.Fujifilm.HighlightTone                  SLong       1  -2
Exif.Fujifilm.Rating                         Long        1  0
"""),
("FujiTagsSTone-2HTone4.jpg",
"""Exif.Fujifilm.ShadowTone                     SLong       1  -2
Exif.Fujifilm.HighlightTone                  SLong       1  +4
Exif.Fujifilm.Rating                         Long        1  0
""")
]

    tags = ["Exif.Fujifilm.ShadowTone",
            "Exif.Fujifilm.HighlightTone",
            "Exif.Fujifilm.Rating",
            "Exif.Fujifilm.DRangePriority",
            "Exif.Fujifilm.DRangePriorityAuto",
            "Exif.Fujifilm.DRangePriorityFixed"]

    tagcmd = "$exiv2 -K " + " -K ".join(tags)
    commands = ["$tagcmd $data_path/" + f for f, _ in filesAndExpectedOutput]
    stdout = [e for _, e in filesAndExpectedOutput]
    stderr = [""] * len(filesAndExpectedOutput)
    retval = [0] * len(filesAndExpectedOutput)



# -*- coding: utf-8 -*-

import system_tests


class TestFuzzedPoC(metaclass=system_tests.CaseMeta):

    url = [
        "https://github.com/Exiv2/exiv2/issues/210",
        "https://github.com/Exiv2/exiv2/issues/209"
    ]

    filename = system_tests.path("$data_path/2018-01-09-exiv2-crash-002.tiff")
    commands = [
        "$exiv2 -pR $filename",
        "$exiv2 -pS $filename",
        "$exiv2 $filename"
    ]
    retval = [1, 1, 0]

    compare_stderr = system_tests.check_no_ASAN_UBSAN_errors

    def compare_stdout(self, i, command, got_stdout, expected_stdout):
        """ We don't care about the stdout, just don't crash """
        pass



# -*- coding: utf-8 -*-

import system_tests


class EncodingTest(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/460"

    filename = "$data_path/exiv2-bug460.jpg"

    commands = ["$exiv2 -u -pt $filename"]

    stdout = ["""Exif.Image.XPTitle                           Byte       40  Titel und ähnliches
Exif.Image.XPComment                         Byte       46  And a standard comment
Exif.Image.XPAuthor                          Byte       44  Українська (Russian?)
Exif.Image.XPKeywords                        Byte       30  עברית (Hebrew)
Exif.Image.XPSubject                         Byte       32  తెలుగు (Telugu)
"""]
    stderr = [""]
    retval = [0]



import system_tests


class XMPMetaDeleteNamespace(
        metaclass=system_tests.CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/984
    """
    url = "https://github.com/Exiv2/exiv2/issues/984"

    filename = system_tests.path(
        "$data_path/issue_94_poc3.pgf"
    )
    commands = ["$exiv2json $filename"]
    stdout = ["""Caught Exiv2 exception 'XMP Toolkit error 9: Fatal namespace map problem'
"""]
    stderr = ["""Warning: Removing 54 characters from the beginning of the XMP packet
"""]
    retval = [1]



# -*- coding: utf-8 -*-

import system_tests

class CheckOptionK(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1053"

    filename = system_tests.path("$data_path/exiv2-bug884c.jpg")
    commands = [ "$exiv2 -PE -g ImageWidth            $filename",
        "$exiv2 -PE -K ImageWidth            $filename",
        "$exiv2 -PE -K Exif.Image.ImageWidth $filename" ]

    stdout = [ """Exif.Image.ImageWidth                        Long        1  3040
""",
"",
"""Exif.Image.ImageWidth                        Long        1  3040
""",
    ]
    stderr = [""] * 3
    retval = [0, 1, 0]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, path, check_no_ASAN_UBSAN_errors


class InfiniteRecursionInCiffDirectoryReadDirectory(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/issues/1097

    A malicious file can cause an infinite recursion.
    """
    url = "https://github.com/Exiv2/exiv2/issues/1097"

    filename = path("$data_path/issue_1097_poc.crw")
    commands = ["$exiv2 $filename"]
    stdout = [""]
    stderr = [
        """$exiv2_exception_message $filename:
$kerOffsetOutOfRange
"""]
    retval = [1]



# -*- coding: utf-8 -*-

from system_tests import CaseMeta, CopyTmpFiles, path, check_no_ASAN_UBSAN_errors
import unittest

@unittest.skip("Skipping test using option -pR (only for Debug mode)")
class ImagePrintIFDStructureZeroCountAssert(metaclass=CaseMeta):
    """
    Regression test for the bug described in:
    https://github.com/Exiv2/exiv2/security/advisories/GHSA-g44w-q3vm-gwjq
    """
    url = "https://github.com/Exiv2/exiv2/security/advisories/GHSA-g44w-q3vm-gwjq"

    filename = path("$data_path/issue_ghsa_g44w_q3vm_gwjq_poc.jpg")
    commands = ["$exiv2 -pR $filename"]
    stderr = ["""invalid type in tiff structure0
Exiv2 exception in print action for file $filename:
$kerInvalidTypeValue
"""]
    retval = [1]

    compare_stdout = check_no_ASAN_UBSAN_errors



# -*- coding: utf-8 -*-

import system_tests

@system_tests.CopyFiles("$data_path/exiv2-bug1202.jpg")
class CheckFocusContinuous(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1202"
    filename = "$data_path/exiv2-bug1202_copy.jpg"

    commands = [
        """$exiv2 -M"set Exif.CanonCs.FocusContinuous SShort 0" $filename""",
        """$exiv2 -K Exif.CanonCs.FocusContinuous $filename""",

        """$exiv2 -M"set Exif.CanonCs.FocusContinuous SShort 1" $filename""",
        """$exiv2 -K Exif.CanonCs.FocusContinuous $filename""",

        """$exiv2 -M"set Exif.CanonCs.FocusContinuous SShort 8" $filename""",
        """$exiv2 -K Exif.CanonCs.FocusContinuous $filename""",

        """$exiv2 -M"set Exif.CanonCs.FocusContinuous SShort 9" $filename""",
        """$exiv2 -K Exif.CanonCs.FocusContinuous $filename""",

        """$exiv2 -M"set Exif.CanonCs.FocusContinuous SShort -1" $filename""",
        """$exiv2 -K Exif.CanonCs.FocusContinuous $filename""",
    ]

    stdout = [
        "",
        "Exif.CanonCs.FocusContinuous                 Short       1  Single\n",

        "",
        "Exif.CanonCs.FocusContinuous                 Short       1  Continuous\n",

        "",
        "Exif.CanonCs.FocusContinuous                 Short       1  Manual\n",

        "",
        "Exif.CanonCs.FocusContinuous                 Short       1  (9)\n",

        "",
        "Exif.CanonCs.FocusContinuous                 Short       1  (65535)\n",
    ]
    stderr = [""] * len(stdout)
    retval = [0] * len(stdout)



# -*- coding: utf-8 -*-

import system_tests

@system_tests.CopyFiles("$data_path/exiv2-bug1112.xmp")
class CheckXmpTimeZoneInformation(metaclass=system_tests.CaseMeta):

    url = "http://dev.exiv2.org/issues/1112"

    filename = system_tests.path("$data_path/exiv2-bug1112_copy.xmp")
    commands = [ """$exiv2 -M "del Xmp.dc.title" $filename""" ]

    stdout = [ "" ]
    stderr = [""]
    retval = [0]

    xmp_packet = """<?xpacket begin="﻿" id="W5M0MpCehiHzreSzNTczkc9d"?>
<x:xmpmeta xmlns:x="adobe:ns:meta/" x:xmptk="XMP Core 4.4.0-Exiv2">
 <rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">
  <rdf:Description rdf:about=""
    xmlns:xmp="http://ns.adobe.com/xap/1.0/"
   xmp:CreateDate="2012-02-01T16:28:00+02:00"/>
 </rdf:RDF>
</x:xmpmeta>
<?xpacket end="w"?>"""

    def post_tests_hook(self):
        with open(self.filename, "r", encoding='utf-8') as xmp_file:
            self.assertMultiLineEqual(self.xmp_packet, xmp_file.read(-1))
